/*
 * daemon fuer i4lmon (windows)
 *
 * Nimmt folgende Kommandos, es muessen immer 2 Texte getrennt durch
 * Leerzeichen angegeben werden.
 *
 * list if       : liefert die Namen aller vorhandenen ippp-interfaces in einer
 *                 Zeile getrennt durch Leerzeichen. Als erstes Wort der
 *                 Zeile wird "Ok" ausgegeben. z.B.
 *                 "Ok ippp0 ippp1 ippp2 ippp3"
 *
 * list if2      : analog zu list if zeigt aber nur interfaces mit slaves
 *
 * list <Name>   : liefert die Einstellungen (=isdnctrl list name)
 *                 Es werden mehrere Zeilen ausgegeben. Die letzte Zeile
 *                 beginnt mit !ERROR oder mit Ok
 *
 * status <Name> : liefert ab Kernel 2.2.13 den Status des mit Name angegebenen
 *                 Interface, liefert:
 *                 "1 I Nummer" : Verbunden, Incoming mit Nummer
 *                 "1 O Nummer" : Verbunden, Outgoing mit Nummer
 *                 "0"          ; nicht verbunden
 *                 und !ERROR blabla.. falls der Kernel kein Status kann
 *
 * dial <Name>   : waehlen fuer das mit <Name> angegebene Interface
 *
 * hangup <Name> : auglegen fuer das mit <Name> angegebene Interface
 *
 * addlink       : waehlen des Slave-Interface von <Name>
 *
 * removelink    : aufhaengen des Slave Interface von <Name>
 *
 * quit          : beenden
 *
 * ver           : liefert Version von i4lmond
 *
 * numin  <Name> : liefert die konfigurierten eingehenden Nummern
 * numout <Name> : liefert die konfigurierten ausgehenden Nummern
 *
 * getphonebook num : liefert den Namen fuer eine Nummer, i4lmond muss
 *                    mit -p phonebookname gestartet werden. In der angegeben
 *                    Datei stehen Nummern und Namen mit tab getrennt.
 *                    (siehe man imon)
 *
 *
 * Nach einem Connect meldet sich i4lmond mit:
 * READY i4lmond <Version>
 *
 * Erfolg wird als
 * Ok
 * und Fehler als
 * !ERROR [Text]
 * zuruckgegeben
 *
 * Ich habe keine Ahnung von c und keine Ahnung von Programmierung unter
 * Linux (bis ein Pascal Mensch)
 * Daher ist der Code hier ein bischen holperig. Viel habe ich einfach
 * aus einem alten isdnctrl geklaut.
 *
 * todo:
 * - scanf ersetzen, ich habe auf anhieb nix passendes gefunden (ev fgets?)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: i4lmond.c,v $
 * Revision 1.19 2000/07/03 20:49:02  armin
 * /dev/isdn/isdnctrl for Kernels 2.4
 * Revision 1.18 2000/06/06 22:13:21  armin
 * status for kernels > 2.2.13
 * Revision 1.17 2000/03/20 ?
 * make it working with telnet
 * started --login to authenticate user
 * Revision 1.16a 2000/03/07 torsten evers
 * removed compiler warnings due to missing 'return' statements
 * added setphonebook() for new entries
 * Revision 1.16  1999/04/04 16:00:00  armin
 * get command-string manual (not with fgets) to allow cr or lf as EOL
 * char. (to test i4lmond with telnet)
 * Revision 1.15  1999/02/14 22:00:00  christof
 * added mppp-support
 * Revision 1.14  1998/11/15 12:02:00  armin
 * problems if more than one Number in getphonebook, now only the name for
 * the first number is returned
 * dialmode, help, add/delete numbers
 * Revision 1.13  1998/10/06 18:40:00  armin
 * coredump on blank lines in phonebook-file
 * Revision 1.02  1998/10/05 19:30:00  armin
 * INF_DV not defined in old i4l-Versions, deleted because it was not needed
 * coredump if phonenumber or name too long (this is also a Problem in imon)
 *
 * Revision 1.01  1998/09/27 12:00:00  armin
 * list interface/check dial in/out
 *
 * Revision 1.00  1998/09/26 15:45:00  armin
 * Initial revision
 *
 */

#undef  ISDN_DEBUG_MODEM_SENDOPT

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <linux/isdn.h>
#include <linux/isdnif.h>

#ifdef Debug
#include <syslog.h>
#endif

#define version "1.19"

// ----------------------------------------------------------------------
// Phonebook, geklaut aus imon
// ----------------------------------------------------------------------

#define MAX_NAME_LEN 30
#define MAX_PHONE_LEN 30

struct phone_entry {
   struct phone_entry *next;
   char phone[MAX_PHONE_LEN];
   char name[MAX_NAME_LEN];
};

static char *phonebook;
static struct phone_entry *phones;

static int Star(char *s, char *p);
/*
 * Wildmat routines
 */
static int wildmat(char *s, char *p) {
  register int   last;
  register int   matched;
  register int   reverse;

  for ( ; *p; s++, p++)
    switch (*p) {
    case '\\':
      /* Literal match with following character; fall through. */
      p++;
    default:
      if (*s != *p)
        return(0);
      continue;
    case '?':
      /* Match anything. */
      if (*s == '\0')
        return(0);
      continue;
    case '*':
      /* Trailing star matches everything. */
      return(*++p ? Star(s, p) : 1);
    case '[':
      /* [^....] means inverse character class. */
      if ((reverse = (p[1] == '^')))
        p++;
      for (last = 0, matched = 0; *++p && (*p != ']'); last = *p)
        /* This next line requires a good C compiler. */
        if (*p == '-' ? *s <= *++p && *s >= last : *s == *p)
          matched = 1;
      if (matched == reverse)
        return(0);
      continue;
    }
  return(*s == '\0');
}

static int Star(char *s, char *p) {
  while (wildmat(s, p) == 0)
    if (*++s == '\0')
      return(0);
  return(1);
}

/*
 * Read phonebook file and create a linked
 * list of phone entries.
 */
static void readphonebook() {
  FILE *f;
  struct phone_entry *p;
  struct phone_entry *q;
  char line[255];
  char *s;

  if (!(f = fopen(phonebook, "r"))) {
    fprintf(stderr, "!ERROR Can't open %s\n",phonebook);
  }
  p = phones;
  while (p) {
    q = p->next;
    free(p);
    p = q;
  }
  phones = NULL;
  while (fgets(line,sizeof(line),f)) {
    if ((s = strchr(line,'\n')))
      *s = '\0';
    if (line[0]=='#')
      continue;
    s = strchr(line,'\t');
    if (s) {  // ad 6.10.98
      *s++ = '\0';
      if (strlen(line) && strlen(s)) {
//        fprintf (stderr,"%s %s\n",line,s);
        q = (struct phone_entry *)malloc(sizeof(struct phone_entry));  // so oder besser (char *)q ?
        q->next = phones;
        if (strlen (line) > MAX_PHONE_LEN) {  // AD 5.10.1998
          line [MAX_PHONE_LEN] = '\0';
        }
        strcpy(q->phone,line);

        if (strlen (s) > MAX_NAME_LEN) {  // AD 5.10.1998
          *(s+MAX_NAME_LEN) = '\0';       // corrected MAX_PHONE_LEN -> MAX_NAME_LEN 07.03.2000 TE
        }

        strcpy(q->name,s);
        phones = q;
      }
    }
  }
  fclose(f);
}

static void setphonebook(char *number,char *name)
{
    struct phone_entry *p;
    struct phone_entry *q;
    FILE *f;

  if (phonebook==NULL)
  {
    fprintf(stderr,"!ERROR no phonebook given !\n");
    return;
  }
  if (!(f=fopen(phonebook,"w"))) {
    fprintf(stderr,"!ERROR could not open/create %s\n",phonebook);
    return;
  }
  if (strlen(number)>MAX_PHONE_LEN)
    number[MAX_PHONE_LEN]='\0';
  if (strlen(name)>MAX_NAME_LEN)
    name[MAX_NAME_LEN]='\0';
  q=(struct phone_entry *)malloc(sizeof(struct phone_entry));
  q->next=phones;
  strcpy(q->phone,number);
  strcpy(q->name,name);
  phones=q;
  p=q;
  while(p)
  {
        fprintf(f,"%s\t%s\n",p->phone,p->name);
        p=p->next;
  }
  fclose(f);
}

/*
 * Find a phone number in list of phone entries.
 * If found, return corresponding name, else
 * return number string.
 */
char *find_name(char *num) {
  struct phone_entry *p = phones;
  static char tmp[100];

  sprintf(tmp, "%-28s", num);
  while (p) {
    if (wildmat(num, p->phone)) {
      sprintf(tmp, "%-28s", p->name);
      break;
    }
    p = p->next;
  }
  return tmp;
}

// ----------------------------------------------------------------------
// Rest ist aus isdnctrl
// ----------------------------------------------------------------------

char nextlistif[10];
char *l2protostr[] = {"x75i", "x75ui", "x75bui", "hdlc", "\0"};
int  l2protoval[]  = {ISDN_PROTO_L2_X75I, ISDN_PROTO_L2_X75UI,
                      ISDN_PROTO_L2_X75BUI, ISDN_PROTO_L2_HDLC, -1};

char *l3protostr[] = {"trans", "\0"};
int  l3protoval[]  = {ISDN_PROTO_L3_TRANS, -1};

char *pencapstr[]  = {"ethernet", "rawip", "ip", "cisco-h", "syncppp" ,
                      "uihdlc", "\0"};
int  pencapval[]   = {ISDN_NET_ENCAP_ETHER, ISDN_NET_ENCAP_RAWIP,
                      ISDN_NET_ENCAP_IPTYP, ISDN_NET_ENCAP_CISCOHDLC,
                      ISDN_NET_ENCAP_SYNCPPP , ISDN_NET_ENCAP_UIHDLC, -1};

char *num2callb[]  = {"off","in","out"};

char * num2key(int num, char **keytable, int *numtable)
{
        int i = -1;
        while (numtable[++i] >= 0)
                if (numtable[i] == num)
                        return keytable[i];
        return "???";
}

static void listbind(char *s, int e)
{
        if (strlen(s)) {
                char *p = strchr(s, ',');
                int ch;
                sscanf(p + 1, "%d", &ch);
                *p = '\0';
                fprintf(stderr,"%s, Channel %d%s\n", s, ch, (e > 0) ? ", exclusive" : "");
        } else
                fprintf(stderr,"Nothing\n");
}

int data_version;

void check_version () {
        int fd;

        fd = open("/dev/isdn/isdninfo", O_RDWR);
	if (fd < 0)
  	  fd = open("/dev/isdninfo", O_RDWR);
        if (fd < 0) {
                fprintf (stderr, "!ERROR opening /dev/isdninfo or /dev/isdn/isdninfo\n");
                exit(-1);
        }
        data_version = ioctl(fd, IIOCGETDVR, 0);
        if (data_version < 0) {
                fprintf(stderr, "!ERROR Could not get version of kernel ioctl structs!\n");
                exit(-1);
        }
        close(fd);
        data_version = (data_version >> 8) & 0xff;
        if (data_version != NET_DV) {
                fprintf(stderr, "!ERROR Version of kernel ioctl structs (%d) does NOT match\n", data_version);
                if (data_version < 1) {
                        fprintf(stderr, "!ERROR Kernel-Version too old, terminating.\n");
                        exit(-1);
                }
                if (data_version > NET_DV) {
                        fprintf(stderr, "!ERROR Kernel-Version newer than i4lmon-Version, terminating.\n");
                        exit(-1);
                }
                if ((NET_DV == 3) || (data_version == 3)) {
                        fprintf(stderr, "!ERROR Version 3 is an interim NOT compatible to others, terminating. RECOMPILE i4lmond\n");
                        exit(-1);
                }
                /*
                if (data_version < 3)
                        fprintf(stderr, "- Option 'trigger' disabled.\n");
                if (data_version < 2)
                        fprintf(stderr, "- Option 'chargeint' disabled.\n");
                */
        }
}

static void listif(int isdnctrl, char *name, int errexit)
{
        isdn_net_ioctl_cfg cfg;
        union p {
                isdn_net_ioctl_phone phone;
                char n[1024];
        } ph;
        char nn[1024];

        strcpy(cfg.name, name);
        if (ioctl(isdnctrl, IIOCNETGCF, &cfg) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGCF) on %s\n",name);
                if (errexit) {
                        exit(-1);
                } else
                        return;
        }
        strcpy(ph.phone.name, name);
        ph.phone.outgoing = 0;
        if (ioctl(isdnctrl, IIOCNETGNM, &ph.phone) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGNM) on %s\n",name);
                if (errexit) {
                        exit(-1);
                } else
                        return;
        }
        strcpy(nn, ph.n);
        strcpy(ph.phone.name, name);
        ph.phone.outgoing = 1;
        if (ioctl(isdnctrl, IIOCNETGNM, &ph.phone) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGNM) on %s\n",name);
                if (errexit) {
                        exit(-1);
                } else
                        return;
        }
        fprintf(stderr,"\nCurrent setup of interface '%s':\n\n", cfg.name);
        fprintf(stderr,"EAZ/MSN:                %s\n", cfg.eaz);
        fprintf(stderr,"Phone number(s):\n");
        fprintf(stderr,"  Outgoing:             %s\n", ph.n);
        fprintf(stderr,"  Incoming:             %s\n", nn);
#ifdef ISDN_NET_DM_OFF
        fprintf(stderr,"Dial mode:              ");
        if (cfg.dialmode == ISDN_NET_DM_OFF)
                fprintf(stderr,"off\n");
        else if (cfg.dialmode == ISDN_NET_DM_AUTO)
                fprintf(stderr,"auto\n");
        else
                fprintf(stderr,"manual\n");
#else
#warning ISDN_NET_DM_OFF not defined? Old isdn4kernel?
        printf("Dial mode:              not available at compilation (Old isdn4kernel?)\n");
#endif

#ifdef I4L_CTRL_TIMRU
#ifdef HAVE_TIMRU
        fprintf(stderr,"Status:                 %s\n", cfg.stopped ? "off" : "on");
#else
        fprintf(stderr,"Status:                 No TIMRU in Kernel\n");
#endif
#endif
        fprintf(stderr,"Secure:                 %s\n", cfg.secure ? "on" : "off");
        fprintf(stderr,"Callback:               %s\n", num2callb[cfg.callback]);
        if (cfg.callback == 2)
                fprintf(stderr,"Hangup after Dial       %s\n", cfg.cbdelay ? "on" : "off");
        else
                fprintf(stderr,"Reject before Callback: %s\n", cfg.cbhup ? "on" : "off");
        fprintf(stderr,"Callback-delay:         %d\n",cfg.cbdelay / 5);
        fprintf(stderr,"Dialmax:                %d\n", cfg.dialmax);
#ifdef HAVE_TIMRU
        fprintf(stderr,"Dial-Timeout:           %d\n", cfg.dialtimeout);
        fprintf(stderr,"Dial-Wait:              %d\n", cfg.dialwait);
#endif
        fprintf(stderr,"Hangup-Timeout:         %d\n", cfg.onhtime);
        fprintf(stderr,"Incoming-Hangup:        %s\n", cfg.ihup ? "on" : "off");
        fprintf(stderr,"ChargeHangup:           %s\n", cfg.chargehup ? "on" : "off");
        fprintf(stderr,"Charge-Units:           %d\n", cfg.charge);
        if (data_version < 2)
            fprintf(stderr,"Charge-Interval:        n.a.\n");
        else
        fprintf(stderr,"Charge-Interval:        %d\n", cfg.chargeint);
        fprintf(stderr,"Layer-2-Protocol:       %s\n", num2key(cfg.l2_proto, l2protostr, l2protoval));
        fprintf(stderr,"Layer-3-Protocol:       %s\n", num2key(cfg.l3_proto, l3protostr, l3protoval));
        fprintf(stderr,"Encapsulation:          %s\n", num2key(cfg.p_encap, pencapstr, pencapval));
        fprintf(stderr,"Slave Interface:        %s\n", strlen(cfg.slave) ? cfg.slave : "None");
        fprintf(stderr,"Slave delay:            %d\n", cfg.slavedelay);
        if (data_version < 3)
            fprintf(stderr,"Slave trigger:          n.a.\n");
#if HAVE_TRIGGERCPS
        else
    	    fprintf(stderr,"Slave trigger:          %d cps\n", cfg.triggercps);
#endif
        fprintf(stderr,"Master Interface:       %s\n", strlen(cfg.master) ? cfg.master : "None");
        fprintf(stderr,"Pre-Bound to:           ");
        listbind(cfg.drvid, cfg.exclusive);
        fprintf(stderr,"PPP-Bound to:           ");
        if (cfg.pppbind >= 0)
                fprintf(stderr,"%d\n", cfg.pppbind);
        else
                fprintf(stderr,"Nothing\n");

        if (cfg.slave && strlen(cfg.slave))
                strcpy(nextlistif, cfg.slave);
        else
                nextlistif[0] = '\0';
        fprintf(stderr,"Ok\n");
}

static void getdialmode(int isdnctrl, char *name)
{
        isdn_net_ioctl_cfg cfg;

        strcpy(cfg.name, name);
        if (ioctl(isdnctrl, IIOCNETGCF, &cfg) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGCF) on %s\n",name);
                return;
        }
#ifdef ISDN_NET_DM_OFF
        if (cfg.dialmode == ISDN_NET_DM_OFF)
                fprintf(stderr,"Ok off\n");
        else if (cfg.dialmode == ISDN_NET_DM_AUTO)
                fprintf(stderr,"Ok auto\n");
        else
                fprintf(stderr,"Ok manual\n");
#else
        fprintf(stderr, "!ERROR not available at compilation (Old isdn4kernel?)\n");
#endif
}

static void setdialmode(int isdnctrl, char *name, char * cmd)
{
#ifdef ISDN_NET_DM_OFF
  isdn_net_ioctl_cfg cfg;
  strcpy(cfg.name, name);
  if ((ioctl(isdnctrl, IIOCNETGCF, &cfg)) < 0) {
    fprintf(stderr, "!ERROR ioctrl (IIOCNETGCF) on %s\n",name);
    return;
  }
  if (strcmp(cmd, "manual") && strcmp(cmd, "on") && strncmp(cmd, "auto", 4) &&
      strcmp(cmd, "off")) {
    fprintf(stderr, "!ERROR dialmode must be 'off', 'manual' or 'auto'\n");
    return;
  }
  if (!strcmp(cmd, "on") || !strcmp(cmd, "manual"))
    cfg.dialmode = ISDN_NET_DM_MANUAL;
  else if (!strcmp(cmd, "off"))
    cfg.dialmode = ISDN_NET_DM_OFF;
  else
    cfg.dialmode = ISDN_NET_DM_AUTO;
  if ((ioctl(isdnctrl, IIOCNETSCF, &cfg)) < 0) {
    fprintf(stderr, "!ERROR ioctrl (IIOCNETSCF) on %s\n",name);
    return;
  }
  fprintf (stderr,"Ok\n");
#else   /* not in kernel include file */
  fprintf (stderr, "!ERROR No 'dialmode' field in kernel source when compiled, old isdn4kernel?\n");
#endif
}

static void listnumbers(int isdnctrl, char *name, int outgoing)
{
        isdn_net_ioctl_cfg cfg;
        union p {
                isdn_net_ioctl_phone phone;
                char n[1024];
        } ph;
        char nn[1024];

        strcpy(cfg.name, name);
        if (ioctl(isdnctrl, IIOCNETGCF, &cfg) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGCF) on %s\n",name);
                return;
        }
        strcpy(ph.phone.name, name);
        ph.phone.outgoing = 0;
        if (ioctl(isdnctrl, IIOCNETGNM, &ph.phone) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGNM) on %s\n",name);
                return;
        }
        strcpy(nn, ph.n);
        strcpy(ph.phone.name, name);
        ph.phone.outgoing = 1;
        if (ioctl(isdnctrl, IIOCNETGNM, &ph.phone) < 0) {
                fprintf(stderr, "!ERROR ioctrl (IIOCNETGNM) on %s\n",name);
                return;
        }
        if (outgoing)
          fprintf(stderr,"Ok %s\n", ph.n);
        else
          fprintf(stderr,"Ok %s\n", nn);
}

int dial (int fd, char * ifname)
{
  if ((ioctl(fd,IIOCNETDIL,ifname))<0) {
    return (1);
  } else {
    return (0);
  }
}

int addlink (int fd, char * ifname)
{
  isdn_net_ioctl_cfg cfg;
  strcpy(cfg.name,ifname);
  if (ioctl(fd,IIOCNETGCF,&cfg)>=0) {

    if ((ioctl(fd,IIOCNETDIL,cfg.slave))<0) {
      return (1);
    }
  }
  return (0);
}

int adddelphone (int fd, char * ifname, char * phonenumber, int outgoing, int add)
{
  isdn_net_ioctl_phone phone;
  int request;

  if (add == 1)
    request = IIOCNETANM;
  else
    request = IIOCNETDNM;

  if (strlen(phonenumber) > 20) {
    fprintf(stderr,"!ERROR phone-number must not exceed 20 characters\n");
    return (1);
  }

  strcpy (phone.name, ifname);
  strcpy (phone.phone, phonenumber);
  phone.outgoing = outgoing;

  if ((ioctl(fd, request, &phone)) < 0) {
    fprintf(stderr,"!ERROR deleting phone '%s' for if '%s' (Out:%d)\n",phonenumber,ifname,outgoing);
    return (1);
  } else {
    fprintf(stderr,"Ok\n");
    return (0);
  }
}

int hangup (int fd, char * ifname)
{
  if ((ioctl(fd,IIOCNETHUP,ifname))<0) {
    return (1);
  } else {
    return (0);
  }
}

int removelink (int fd, char * ifname)
{
  isdn_net_ioctl_cfg cfg;
  strcpy(cfg.name,ifname);
  if (ioctl(fd,IIOCNETGCF,&cfg)>=0) {
    if ((ioctl(fd,IIOCNETHUP,cfg.slave))<0) {
      return (1);
    }
  }
  return (0);
}

void listippp (int fd)
{
FILE *iflst;
char *p;
char s[255];
char name[10];
char names[512];
isdn_net_ioctl_cfg   cfg;

    if ((iflst = fopen("/proc/net/dev","r"))==NULL) {
          fprintf (stderr, "!ERROR\n");
          return;
    }
    names[0] = 0;
    while (!feof(iflst)) {
      fgets(s,sizeof(s),iflst);
      if ((p = strchr(s,':'))) {
        *p = 0;
        sscanf(s,"%s",name);

        strcpy(cfg.name,name);
        if (ioctl(fd,IIOCNETGCF,&cfg)>=0) {
          // printf ("Name:%s\n", name); // listif(fd,name,0);
          strcat (names, name);
          strcat (names," ");
        }
      }
    }
    fclose(iflst);
    fprintf (stderr, "Ok %s\n", names);
}

void listbundlingippp (int fd)
{
FILE *iflst;
char *p;
char s[255];
char name[10];
char names[512];
isdn_net_ioctl_cfg   cfg;

    if ((iflst = fopen("/proc/net/dev","r"))==NULL) {
          fprintf (stderr, "!ERROR\n");
          return;
    }
    names[0] = 0;
    while (!feof(iflst)) {
      fgets(s,sizeof(s),iflst);
      if ((p = strchr(s,':'))) {
        *p = 0;
        sscanf(s,"%s",name);

        strcpy(cfg.name,name);
        if (ioctl(fd,IIOCNETGCF,&cfg)>=0 && strlen(cfg.slave)) {
          // printf ("Name:%s\n", name); // listif(fd,name,0);
          strcat (names, name);
          strcat (names," ");
        }
      }
    }
    fclose(iflst);
    fprintf (stderr, "Ok %s\n", names);
}


#ifdef IIOCNETGPN
void statusif(int isdnctrl, char *name)
{
	isdn_net_ioctl_phone phone;
	int rc;
	int isdninfo;

	isdninfo = open("/dev/isdn/isdninfo", O_RDONLY);
	if (isdninfo < 0)
	  isdninfo = open("/dev/isdninfo", O_RDONLY);
	if (isdninfo < 0) {
	    fprintf (stderr, "!ERROR Can't open /dev/isdninfo or /dev/isdn/isdninfo\n");
	    return;
	}
	
	memset(&phone, 0, sizeof phone);
	strncpy(phone.name, name, sizeof phone.name);
	rc = ioctl(isdninfo, IIOCNETGPN, &phone);
	if (rc == 0) {  // Ok, connected
	    close (isdninfo);
	    fprintf (stderr,"Ok 1 %s %s\n",
			name, phone.outgoing?"O":"I", phone.phone);
	    return;
	}
	rc = errno;
	close (isdninfo);
	
	if (errno == ENOTCONN) {
	    fprintf (stderr,"Ok 0\n");  // not Connected
	    return;
	}
	if (errno == EINVAL) {
	    fprintf (stderr,"!ERROR Sorry, Status not available in your kernel (2.2.12 or higher is required)\n");
	}
}
#else
void statusif(int isdnctrl, char *name)
{
    fprintf (stderr,"!ERROR Sorry, Status not available in your kernel (2.2.12 or higher is required)\n");
}
#endif




void help (void)
{
  fprintf (stderr, "list if                       - Get List of Isdn-Interfaces\n");
  fprintf (stderr, "list if2                      - Get List of mppp-Interfaces\n");
  fprintf (stderr, "list interface                - Get List Interface-Params\n");
  fprintf (stderr, "dial interface                - Force Dial of interface\n");
  fprintf (stderr, "hangup interface              - Force Hangup of interface\n");
  fprintf (stderr, "addlink interface             - Force Dial of slave-interface\n");
  fprintf (stderr, "removelink interface          - Force Hangup of slave-interface\n");
  fprintf (stderr, "numin interface               - List incoming Numbers for interface\n");
  fprintf (stderr, "numout interface              - List outgoing Numbers for interface\n");
  fprintf (stderr, "ver                           - Show Version\n");
  fprintf (stderr, "getphonebook number           - Get Name from Phonebook\n");
  fprintf (stderr, "setphonebook number name      - Make new entry in the Phonebook\n");
  fprintf (stderr, "addphoneout if num            - Add outgoing phonenumber num to interface if\n");
  fprintf (stderr, "addphonein if num             - Add incoming phonenumber num to interface if\n");
  fprintf (stderr, "delphoneout if num            - Delete outgoing number num from interface if\n");
  fprintf (stderr, "delphonein if num             - Delete incoming number num from interface if\n");
#ifdef ISDN_NET_DM_OFF
  fprintf (stderr, "dialmode if [off|manual|auto] - set the dial mode\n");
  fprintf (stderr, "dialmode if                   - show Dialmode for interface if\n");
#endif
  fprintf (stderr, "status if                     - show connection status, 0=no,1=connected I|O Number\n");

  fprintf (stderr, "\n");
  fprintf (stderr, "On Errors the first 6 Characters of the output-Line will be !ERROR\n");
  fprintf (stderr, "Otherwise the first 2 Characters of the ouput-Line will be Ok\n");
  fprintf (stderr, "Lines that do not begin with Ok or !ERROR are Result-Lines\n");
  fprintf (stderr, "Ok\n");
}

int readline (char *s, int maxlen)
{
  int len = 0;
  char c;
  *s = '\0';
  c = getchar();
  while ((len < maxlen) && (c != '\n') && (c != '\r')) {
    *s = c;
    s++;
    *s = '\0';
    len++;
    c = getchar();
  }
  return len;
}


int check_login(int num, char **args) {
  char user[128], pass[128], *p;
  int  cnt = num;

  for(cnt--; cnt > 0; cnt--) {
    if(!strcmp(args[cnt], "--login-check")) {
      cnt = 0;
      fprintf(stdout, "please enter user name : ");
      fgets(user, sizeof(user), stdin);
      if((p = strrchr(user, '\n')) != NULL) *p = '\0';
      if((p = strrchr(user, '\r')) != NULL) *p = '\0';
      if(strlen(user)) {
        fprintf(stdout, "please enter password : ");
        fgets(pass, sizeof(pass), stdin);
        if((p = strrchr(pass, '\n')) != NULL) *p = '\0';
        if((p = strrchr(pass, '\r')) != NULL) *p = '\0';
        return(1);
      } 
      fprintf(stdout, "!ERROR : empty user name !!!\n");
      return(0);
    }
  }
  return(1);
}



int main(int argc, char **argv) {
  int fd;
  char command [255];
  char * param;
  char * c;
  char * ifname;
  char * phone;
  
  int cmd_only;
  int end = 0;
  char cmd [255];

  if(check_login(argc, argv) != 1) exit(1);
  check_version();

  fd = open("/dev/isdn/isdnctrl",O_RDWR);
  if (fd < 0)
    fd = open("/dev/isdnctrl",O_RDWR);
  if (fd < 0) {
    fprintf (stderr, "!ERROR opening /dev/isdnctrl or /dev/isdn/isdnctrl\n");
    exit(-1);
  }
  phonebook=NULL;     // make sure not existing/given phonebook is not opened for write */
  if (argc > 1) {
   if (argv[1][0] == '-') {
    if (argv[1][1] == 'p') {
      phonebook = argv[2];
      readphonebook();
    }
   }
  }

  fprintf (stderr, "READY i4lmond %s\n", version);
#ifdef Debug
  syslog (LOG_INFO,"READY i4lmond %s\n", version);
#endif
  while (!end) {
    /* scanf ("%s %s", &command, &param); */
    command [0] = '\0';
    cmd_only = 0;
    fgets (&command[0], sizeof (command), stdin);

    param = &command[0];
    strcpy(cmd, command);
    if((c = strrchr(cmd, '\r')) != NULL) *c = '\0';
    if((c = strrchr(cmd, '\n')) != NULL) *c = '\0';

    while ((*param != '\0') && (*param != ' ')) {
      if (*param == '\n' || *param == '\r') {  /* strip CR and newline */
        *param = '\0'; }
      else {
        param++;
      }
    }
    if (*param == ' ') {
      *param = '\0';
      param++;
    } else cmd_only = 1;
    if(!cmd_only) {
      while (*param == ' ') param++;
    
      c = strchr (param, '\r');
      if (c != NULL) *c = '\0';  /* strip CR */
      c = strchr (param, '\n');
      if (c != NULL) *c = '\0';  /* strip newline */
    }

#ifdef Debug
    syslog (LOG_INFO,
                "command: '%s', param: '%s'", command,param);
#endif
    if (!strcmp(command,"help")) {
      help ();
    } else
    if (!strcmp(command,"list")) {
      if (!strcmp(param,"if")) {
        listippp (fd);
      } else
      if (!strcmp(param,"if2")) {
        listbundlingippp (fd);
      } else
      {
        listif (fd, param, 0);
      }
    } else
    if (!strcmp(command,"status")) {
      statusif (fd, param);
    } else
    if (!strcmp(command,"dial")) {
      if (dial (fd, param) == 0) {
        fprintf (stderr, "Ok\n");
      } else
      {
        fprintf (stderr, "!ERROR\n");
      }
    } else
    if (!strcmp(command,"hangup")) {
      if (hangup (fd, param) == 0) {
        fprintf (stderr, "Ok\n");
      } else
      {
        fprintf (stderr, "!ERROR\n");
      }
    } else
    if (!strcmp(command,"addlink")) {
      if (addlink (fd, param) == 0) {
        fprintf (stderr, "Ok\n");
      } else
      {
        fprintf (stderr, "!ERROR\n");
      }
    } else
    if (!strcmp(command,"removelink")) {
      if (removelink (fd, param) == 0) {
        fprintf (stderr, "Ok\n");
      } else
      {
        fprintf (stderr, "!ERROR\n");
      }
    } else
    if (!strcmp(command,"numin")) {
      listnumbers (fd, param, 0);
    } else
    if (!strcmp(command,"numout")) {
      listnumbers (fd, param, 1);
    } else
    if (!strcmp(command,"quit")) {
      fprintf (stderr, "Ok\n");
      close(fd);
      return 0;
    } else
    if (!strcmp(command,"ver")) {
      fprintf (stderr, "Ok %s\n", version);
    } else
    if (!strcmp(command,"getphonebook")) {
      c = strchr (param, ' ');          // more than one Number,
      if (c != NULL) *c = '\0';         // than use only the first one
      fprintf (stderr, "Ok %s\n", find_name (param));
    } else
    if (!strcmp(command,"addphoneout")) {
      ifname = param; phone = NULL;
      c = strchr (param, ' ');
      if (c != '\0') {
        *c = '\0'; phone = c; phone++;
        adddelphone (fd, ifname, phone, 1, 1);
      } else
        fprintf (stderr, "!ERROR addphoneout <ifname> <number>\n");
    } else
    if (!strcmp(command,"addphonein")) {
      ifname = param; phone = NULL;
      c = strchr (param, ' ');
      if (c != '\0') {
        *c = '\0'; phone = c; phone++;
        adddelphone (fd, ifname, phone, 0, 1);
      } else
        fprintf (stderr, "!ERROR addphonein <ifname> <number>\n");
    } else
    if (!strcmp(command,"delphoneout")) {
      ifname = param; phone = NULL;
      c = strchr (param, ' ');
      if (c != '\0') {
        *c = '\0'; phone = c; phone++;
        adddelphone (fd, ifname, phone, 1, 0);
      } else
        fprintf (stderr, "!ERROR addphoneout <ifname> <number>\n");
    } else
    if (!strcmp(command,"delphonein")) {
      ifname = param; phone = NULL;
      c = strchr (param, ' ');
      if (c != '\0') {
        *c = '\0'; phone = c; phone++;
        adddelphone (fd, ifname, phone, 0, 0);
      } else
        fprintf (stderr, "!ERROR addphonein <ifname> <number>\n");
    } else
    if (!strcmp(command,"dialmode")) {
      ifname = param; phone = NULL;
      c = strchr (param, ' ');
      if (c != '\0') {
        *c = '\0'; phone = c; phone++;
        setdialmode(fd, ifname, phone);
      } else
        getdialmode(fd, ifname);
    } else
    if (!strcmp(command,"setphonebook")) {
    ifname=param; phone=NULL;
    c=strchr(param,' ');
    if (c!='\0') {
        *c='\0'; phone=c; phone++;
        setphonebook(ifname,phone);
        fprintf(stderr,"OK !\n");
        }
    }
    else
    {
      fprintf (stderr, "!ERROR unknown command (%s)\n",command);
    }
  }
}
