/* Test fuer einen moeglichen Ersatz von cat /dev/isdninfo fuer i4lmon

   compile mit gcc isdninfo isdninfo.c
   
   Armin Diehl   19.12.1999:
     erste Version
   Torsten Evers 21.12.1999: 
     Erweiterung zur Auswertung von /var/log/messages -> Rufnummern
   Armin Diehl   24.12.1999:
     Service-Indikator, ersteinmal fix auf Telefonie da die Zeilen mit
     "isdn_tty: call from" scheinbar nur bei Telefonie kommen
   Torsten Evers 01.02.2000:
     Umstellung auf parent und child prozess mit direktem lesen der /var/log/messages
     Workaround fuer logrotate
   Armin Diehl 13.02.2000:
     Debug-printf's entfernt 
     Fehler ueber syslog
     Daemon beim start
     in watchMessages muss tv vor jedem select initialisiert werden sonst
     100% Systemauslastung
   
*/

/*
Beispiel Kernel 2.0.37
Telefon:
Dec 24 13:43:01 netlinux kernel: ICALL (net): 0 1 002151503999
Dec 24 13:43:01 netlinux kernel: isdn_tty: call from 002151503999 -> 480 ignored
Dec 24 13:43:05 netlinux kernel: ICALL (net): 0 1 002151503999
Dec 24 13:43:05 netlinux kernel: isdn_tty: call from 002151503999 -> 480 ignored

Daten:
Dec 22 15:39:21 netlinux kernel: ICALL (net): 0 0 002115590442
Dec 22 15:39:21 netlinux kernel: ippp18: call from 002115590442 -> 480 accepted
Dec 22 15:39:21 netlinux kernel: DCONN: 0
Dec 22 15:39:21 netlinux kernel: BCONN: 0
Dec 22 15:39:21 netlinux kernel: isdn_net: ippp18 connected

Beispiel Kernel 2.2.13
Telefon:
Dec 24 13:45:54 mail kernel: isdn_net: call from 002151503999,1,0 -> 19
Dec 24 13:45:54 mail kernel: isdn_net: Service-Indicator not 7, ignored
Dec 24 13:45:54 mail kernel: isdn_tty: call from 002151503999 -> 19 ignored

Daten:
Dec 24 13:52:22 mail kernel: isdn_net: call from 25,7,0 -> 21
Dec 24 13:52:22 mail kernel: isdn_net: ippp2 connected
*/

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#define LISTEN_PORT 6105            // spaeter auf 6105 aendern
#define MAX_CLIENTS 100
#define FIRST_CLIENT_SOCKET 3       // 0:listen socket, 1=isdninfo, 2=pipe
#define TIMEOUT_SECS 10

// in diesem Array ist
// 0: der Socket fuer neue Verbindungen
// 1: der Socket auf /dev/isdninfo
// 2: stdin (aus tail -f -n1 /var/log/messages | isdninfo)
// 3.. die Client-Sockets

int wait_sockets [MAX_CLIENTS + FIRST_CLIENT_SOCKET + 1];  // 0=Listen Socket, 1=isdninfo, 2=messages

// am ende alle Sockets schliessen
void termination_handler (int signum)
{
  int j;
    
  for (j = 0; j < MAX_CLIENTS + FIRST_CLIENT_SOCKET; j++)
  {
    if ((j != 2) && (wait_sockets[j] >= 0))
    {  // 2 ist stdin
      close (wait_sockets[j]);
      wait_sockets[j] = -1;
    }
  }
  syslog(LOG_DAEMON | LOG_INFO, "terminated");
  exit (EXIT_SUCCESS);
}

// todo: hier noch die relevanten Zeilen ausfiltern
// DONE !

int reformatMessagesLine (char * line, char * outputLine)
{
   register int i;
   char a[10],b[5],c[15],dummy[5],msn[50],number[50];

   strcpy (outputLine, "1");   // Zeilenkennung
   i=0;
   if (strlen(line)<21) return 0; /* Kann nicht den String enthalten */
   while(i<(strlen(line)-19))
   {
      if (strncmp(&line[i],"isdn_tty: call from",19)==0)
      {
         i+=20;
         sscanf(&line[i],"%s %s %s",number,dummy,msn);
         if (strcmp(number,"0")==0)
            strcpy(number,"Anonym");
         else
         {
            strcat(outputLine,number);
	          strcat(outputLine," ");
            strcat(outputLine,msn);
	          strcat(outputLine," 0");  /* Service-Indicator */
            strcat(outputLine," ");
            sscanf(line,"%s %s %s",a,b,c);
            strcat(outputLine,c);
            strcat(outputLine," ");
            strcat(outputLine,a);
            strcat(outputLine," ");
            strcat(outputLine,b);
            strcat(outputLine,"\n");
         }
         return 1;
      }
      i++;
   }
  return 0;
}
								    


int waitForConnection (int p_fd)
{
  struct sockaddr_in server_addr;
  struct sockaddr_in client_addr;
  int addrlen;
  int i;
  int j;
  int max_fd;
  fd_set iofds;
  struct timeval timeout;
  int newindex;
  char st [300];
  char st2 [300];
  int stlen;
  FILE *fIsdninfo;
  char isdninfobuf [6][300];
  int tempSocket;
  
  for (i=0; i<MAX_CLIENTS + FIRST_CLIENT_SOCKET + 1; i++)
  {
    wait_sockets[i] = -1;
  }
  
  /* Invoke termination_handler when one of these signals happens */
  signal (SIGTERM, termination_handler);
  signal (SIGINT, termination_handler);
  signal (SIGHUP, termination_handler);
  signal (SIGQUIT, termination_handler);
  
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons (LISTEN_PORT);
  server_addr.sin_addr.s_addr = htonl(inet_addr("0.0.0.0"));
  
  if ((wait_sockets[0] = socket (AF_INET, SOCK_STREAM, 0)) < 0)
  {
    perror ("server: socket");
    syslog(LOG_DAEMON | LOG_ERR, "socket failed, exiting");
    return (1);
  }
  
  // bind the socket
  if (bind (wait_sockets[0], (struct sockaddr *)&server_addr,sizeof (server_addr)) < 0)
  {
    syslog(LOG_DAEMON | LOG_ERR, "bind failed, exiting (may be another copy of isdninfo is already running)");
    return (1);
  }
  
  // Listen on the socket
  if (listen (wait_sockets[0], 1) < 0)
  {
    syslog(LOG_DAEMON | LOG_ERR, "listen failed, exiting");
    return (2);
  }
  
  // open /dev/isdninfo
  fIsdninfo = fopen ("/dev/isdninfo", "r");
  if (fIsdninfo == NULL)
  {
    close (wait_sockets[0]);
    syslog(LOG_DAEMON | LOG_ERR, "cant open /dev/isdninfo, exiting");
    return (3);
  }
  // select funktioniert leider nicht mit normalen Dateien, also
  // tail -f -n1 /var/log/messages | isdninfo
  
  wait_sockets [1] = fileno (fIsdninfo);
  wait_sockets [2] = p_fd; // pipe from child
  
  syslog(LOG_DAEMON | LOG_INFO, "started");
  
  
  // loop: accept connections or send changes to clients
  while (1)
  {
    // collect all sockethandles for select
    FD_ZERO (&iofds);
    max_fd = 0;
    for (i = 0; i < MAX_CLIENTS + FIRST_CLIENT_SOCKET; i++)
    {
      if (wait_sockets[i] != -1)
      {
        if (wait_sockets[i] > max_fd) max_fd = wait_sockets[i];
	      FD_SET (wait_sockets[i], &iofds);
      }
    }
    
    // wait for some change
    timeout.tv_sec  = TIMEOUT_SECS;
    timeout.tv_usec = 0;
    
    if (select (max_fd + 1, &iofds, (fd_set *)0, (fd_set *)0,TIMEOUT_SECS ? &timeout : NULL) <= 0)
    {
      // printf ("timeout\n");
    }
    else
    {
      // is it a new connection request ?
      if (FD_ISSET (wait_sockets[0], &iofds))
      {
        /* printf ("new connection request\n"); */
        newindex = FIRST_CLIENT_SOCKET;
        while ((newindex < MAX_CLIENTS + FIRST_CLIENT_SOCKET) && (wait_sockets[newindex] != -1))
        {
          newindex++;
        }
        if ((newindex != MAX_CLIENTS + FIRST_CLIENT_SOCKET) && (newindex > FIRST_CLIENT_SOCKET-1))
        {
          /* printf ("new client socket %d\n", newindex); */
	        addrlen = sizeof (client_addr);
	        wait_sockets[newindex] = accept (wait_sockets[0],(struct sockaddr *)&client_addr,&addrlen);
      	  if (wait_sockets[newindex] < 0)
          {
		  syslog(LOG_DAEMON | LOG_ERR, "accept failed");
	        }
          else
	        {
		        syslog(LOG_DAEMON | LOG_INFO,
		           "connect from %s %d Slot %d\n",inet_ntoa(client_addr.sin_addr),client_addr.sin_port,newindex);
	          // send isdninfo lines from buffer
	          for (j=0; j<6; j++)
            {
	             write (wait_sockets[newindex], &isdninfobuf[j], strlen (isdninfobuf[j]));
	          }
	        }
        }
        else
        {
          // todo: ?? how do i reject a connection-request ??
          // annehmen und sofort die Verbindung beenden ?
	  // ok
	  addrlen = sizeof (client_addr);
	  tempSocket = accept (wait_sockets[0],(struct sockaddr *)&client_addr,&addrlen);
	  close (tempSocket);
	  syslog(LOG_DAEMON | LOG_ERR,
            "connection refused, no free client slots");
        }
      }
      else
      {
        if ((wait_sockets[1] >= 0) && (FD_ISSET (wait_sockets[1], &iofds)))
        {
           // printf ("/dev/isdninfo changed\n");
	         for (j=0; j<6; j++)
           {
	           isdninfobuf[j][0] = '\0';
	           fgets (&st[0], sizeof(st)-1, fIsdninfo);
	           strcpy (&isdninfobuf[j][0],"0");  // Zeilenkennung
	           strcat (&isdninfobuf[j][0], st);
	           stlen = strlen (isdninfobuf[j]);
	           // printf ("sending isdninfo line %d\n", j);
	           // send line to all clients
	           for (i=FIRST_CLIENT_SOCKET; i < MAX_CLIENTS+FIRST_CLIENT_SOCKET; i++)
             {
	             if (wait_sockets[i] >= 0)
               {
	               write (wait_sockets[i], &isdninfobuf[j], stlen);
	             }
	           }
           }
        }
        if ((wait_sockets[2] >= 0) && (FD_ISSET (wait_sockets[2], &iofds)))
        {
	        st2[0] = '\0';
	        /* alt: stdin: fgets (&st2[0], sizeof(st2)-1, stdin); */
	        read(p_fd,st2,sizeof(st2));
	        if (reformatMessagesLine (st2,st))
          {
	          stlen = strlen (st);
	          // printf ("/var/log/messages changed ('%s')\n", st);
	          if (stlen > 2)
            {
  	          // printf ("sending Messages line\n");
	            // send line to all clients
	            for (i=FIRST_CLIENT_SOCKET; i < MAX_CLIENTS+FIRST_CLIENT_SOCKET; i++)
              {
	              if (wait_sockets[i]>= 0)
                {
	                write (wait_sockets[i], &st, stlen);
	              }
	            }
	          }
	        }
        }
        // may be some client closed connection
      	for (i=FIRST_CLIENT_SOCKET; i < MAX_CLIENTS+FIRST_CLIENT_SOCKET; i++)
        {
	        if (wait_sockets[i] >= 0)
          {
	          if (FD_ISSET (wait_sockets[i], &iofds))
            {
	            // da der client nix senden darf, schliessen wir den socket
	            close (wait_sockets[i]);
	            wait_sockets[i] = -1;
		          syslog(LOG_DAEMON | LOG_INFO, "disconnect Slot %d",i);
	          }
	        }
	      }
      }
    }
  }
  return 0;
}

void watchMessages(int p_fd)
{
	FILE *fMess;
	char NextLine[100];
	struct timeval tv;
	int timer;
	
	timer=0;
	
	fMess=fopen("/var/log/messages","r");
	if (fMess==NULL)
	{
		syslog(LOG_DAEMON | LOG_ERR, "open /var/log/messages failed, exiting");
		return;
	}
	fseek(fMess,0,SEEK_END);	/* ans Ende der Datei */
	
	/* set termination handlers */
	signal (SIGTERM, termination_handler);
  signal (SIGINT, termination_handler);
  signal (SIGHUP, termination_handler);
  signal (SIGQUIT, termination_handler);

	while(1)
	{
		if (fgets(NextLine,sizeof(NextLine)-1,fMess)!=NULL)
			write(p_fd,NextLine,strlen(NextLine));
		tv.tv_sec=0;
		tv.tv_usec=500; /* halbe sekunde warten */
		select(0,NULL,NULL,NULL,&tv);
		
		timer++;
		if (timer==7200)
		{
			/* jede Stunde einmal Datei schliessen und wieder oeffnen, um dem "logrotate"-Problem aus
			   dem Weg zu gehen */
			fclose(fMess);
			fMess=fopen("/var/log/messages","r");
			if (fMess==NULL)
			{
				perror("Konnte /var/log/messages nicht oeffnen !\n");
				return;
			}
			fseek(fMess,0,SEEK_END);
		}
	}
}


void daemonize(void)
{
    pid_t pid;

    pid = fork();
    switch (pid) {
    case -1:
	syslog(LOG_DAEMON | LOG_ERR,
		           "fork: daemonize isdninfo failed, exiting\n");
	exit(EXIT_FAILURE);
    case 0:
	setsid();
//	chdir("/");
	umask(0);
	break;
    default:
	exit(EXIT_SUCCESS);
    }
}




int main(void)
{
  pid_t ClientPID;
  int fds[2];
  
  daemonize();

  /* create unnamed pipe */
  if (pipe(fds)!=0)
  {
  	perror("Konnte Pipe nicht erstellen !");
  	exit(11);
  }

  ClientPID=fork();
  if (ClientPID<0)
  {
     perror("Fehler bei fork !");
     exit(10);
  }
  if (ClientPID==0)                 /* Im Child ! */
  {
     watchMessages(fds[1]);         /* /var/log/messages ueberwachen */
     close(fds[0]);
     exit(0);
  }
  else
  {
    waitForConnection (fds[0]);
    close(fds[1]);
  }
  return(0);
}

