//  $HDR$
// **********************************************************************
//  Armin Diehl (diehl@nordrhein.de)
// **********************************************************************
//  $Log:  \\FS-AD\PRJ\GP-Version\Archives\i4l Monitor\I4lmon\IsdnInfoMon.pas.z 
// 
//    Rev 1.0    24.12.2000 15:22:27  ad
//  Initial Revision

// 
UNIT IsdnInfoMon;

(*
 changelog:
 ad 22.09.1998: auto reconnect and different message for connect,
                disconnect and statuc-change

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 Win32 Verbindungsmonitor (like imon) fr Win32
 12.10.1998 Armin Diehl (diehl@net-connection.de)

 Der Thread verbindet zu Port 6105 des Linux. Dort mu folgendes
 Konfiguriert werden:

 in /etc/services:
 isdninfo         6105/tcp    # /dev/isdninfo

 und in /etc/inetd.conf
 isdninfo   stream tcp   nowait  root  /usr/sbin/tcpd /bin/cat /dev/isdninfo

 danach die Konfiguration mit
 killall -HUP inetd
 neu einlesen. Testen mit
 telnet localhost isdninfo

 Eventuell mu der Service noch in /etc/hosts.allow erlaubt werden.

 Dieser Thread schreibt alle Infos in die Variable IsdnStatus. Bei einer
 nderung wird dem bei create angegebenen Fenster eine Message
 WM_USER+1 gesendet.

 Der Thread wird initialisiert mit
   MyThreadVar := TIsdnInfoThread.Create (LinuxAddrOderName, Hwnd)

 LinuxAddrOderName ist die Ip-Adresse (als Text) des Linuxsystems oder
 ein Hostname (falls in Hosts eingetragen oder ber DNS zu ermitteln)

 Als HWND wird der Fensterhandle fr Benachrichtigung bei einer nderung
 angegeben. In einem TForm ist das einfach Handle.
*)

INTERFACE

USES WinSock, Classes, SysUtils, Forms, Windows, Messages, Grids;

CONST I4LMaxChannelsK  = 64;
      I4LMaxCardsK     = 16;  // only for num of icons in systray
      IsdnMessageK     = WM_USER + 1;
      MsgParam_StatusChangeK = 0;
      MsgParam_DisconnectedK = 1;
      MsgParam_ConnectedK    = 2;  // only after a disconnect, Data in IsdnStatus may not be valid

VAR   LnxPortK       : WORD  = 6105;      

TYPE TIsdnException = CLASS (Exception);
     TIsdnInfo = RECORD   // fr einen Channel
                   IdString              : STRING [30];  // idmap
                   ChannelLowLevelDriver : BYTE;         // chmap
                   IndexLowLevelDriver   : BYTE;         // drmap
                   Usage                 : BYTE;
                   Phone                 : STRING;
                 END;

     TIsdnInfoThread = CLASS (TTHread)
       SockHandle : TSocket;
       SignalWinHandle : HWND;
       LnxIpAddrOrName : STRING;
       CONSTRUCTOR Create (LinuxAddressOrName     : STRING;
                           WinHandle              : HWND);
       DESTRUCTOR Destroy; OVERRIDE;
       PROCEDURE Execute; OVERRIDE;
       FUNCTION  Connect : BOOLEAN;
     END;

VAR IsdnStatus : ARRAY [0..I4LMaxChannelsK] OF TIsdnInfo;
    DriverFlags: ARRAY [0..I4LMaxChannelsK] OF CHAR;
    ConnectionUp : BOOLEAN = FALSE;

// Kommunikation mit i4lmond:

// Returns List of ippp interfaces separated by space
FUNCTION i4ld_GetIppps   (VAR   IpppList : STRING) : BOOLEAN;

// dials interface IpppName
FUNCTION i4ld_Dial       (CONST IpppName : STRING) : BOOLEAN;

// hangup interface IpppName
FUNCTION i4ld_Hangup     (CONST IpppName : STRING) : BOOLEAN;

// connect via tcp to i4lmond
FUNCTION i4ld_Connect    (LinuxAddressOrName     : STRING) : BOOLEAN;

// disconnect from i4lmond
FUNCTION i4ld_DisConnect : BOOLEAN;

// are we connected to i4lmond ?
FUNCTION i4ld_Connected : BOOLEAN;

// Fill info into StGrid
FUNCTION i4ld_ListIppp (CONST IpppName : STRING;
                              StGrid   : TStringGrid) : BOOLEAN;

FUNCTION i4ld_GetOutoingNumbers (CONST IpppName : STRING) : STRING;

FUNCTION i4ld_GetPhonbookEntry (CONST Number : STRING) : STRING;

IMPLEMENTATION


FUNCTION TCPConnectTo (VAR aSocket      : TSocket;
                           AdressOrName : PCHAR;
                           Port         : WORD;
                           SetNDelay    : BOOLEAN) : INTEGER;
VAR Remote_Addr : TSockAddrIn;
    Remote_Host : Phostent;
    A           : TInAddr;
    OptVal      : INTEGER;
BEGIN
  Remote_Addr.sin_addr.S_addr := 0;
  // ersteinmal versuchen in eine Adresse umzuwandeln
  u_long (A) := inet_addr (AdressOrName);
  IF longword (A) = INADDR_NONE THEN
  BEGIN
    // ist eventuell ein Hostname
    Remote_Host := gethostbyname(AdressOrName);
    IF Remote_Host <> NIL THEN
      Remote_Addr.sin_addr := PInAddr(Remote_Host^.h_addr_list^)^;
  END ELSE
    Remote_Addr.sin_addr.S_addr := u_long (A);

  Remote_Addr.sin_family := PF_INET;
  Remote_Addr.sin_port := htons (Port);

  // Ok, Host bekannt, socket holen
  IF Remote_Addr.sin_addr.S_addr = 0 THEN
  BEGIN
    Result := WSAGetLastError;
    IF Result = 0 THEN Result := 1;
    aSocket := INVALID_SOCKET;
    EXIT;
  END ELSE
    aSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);

  IF aSocket = INVALID_SOCKET THEN
  BEGIN
    Result := WSAGetLastError;
    IF Result = 0 THEN INC (Result);
    EXIT;
  END;

  IF connect (aSocket, Remote_Addr, SIZEOF (Remote_Addr)) <> 0 THEN
  BEGIN
    closesocket (aSocket);
    Result := WSAGetLastError;
    aSocket := INVALID_SOCKET;
    IF Result = 0 THEN INC (Result);
    EXIT;
  END;

  IF SetNDelay THEN
  BEGIN
    OptVal := 1;
    setsockopt (aSocket, IPPROTO_TCP, TCP_NODELAY, @OptVal, SIZEOF(OptVal));
  END;
  Result := 0;
END;



CONSTRUCTOR TIsdnInfoThread.Create (LinuxAddressOrName     : STRING;
                                    WinHandle              : HWND);
VAR Err : INTEGER;
BEGIN
  INHERITED Create (TRUE);  // luft noch nicht
  FreeOnTerminate := TRUE;
  SockHandle := -1;
  SignalWinHandle := WinHandle;
  LnxIpAddrOrName := LinuxAddressOrName;
  Err := TCPConnectTo (SockHandle, PChar(LnxIpAddrOrName), LnxPortK, FALSE);
  IF Err <> 0 THEN
    RAISE (TIsdnException.Create ('Connect zu '+LinuxAddressOrName+'/'+IntToStr(LnxPortK)+' fehlgeschlagen, WSA '+IntToStr(WSAGetLastError)));
  Suspended := FALSE;
  ConnectionUp := TRUE;
END;

DESTRUCTOR TIsdnInfoThread.Destroy;
BEGIN
  IF SockHandle <> -1 THEN closesocket (SockHandle);
  ConnectionUp := FALSE;
  INHERITED Destroy;
END;



FUNCTION  TIsdnInfoThread.Connect : BOOLEAN;
VAR Err : INTEGER;
BEGIN
  Err := TCPConnectTo (SockHandle, PChar(LnxIpAddrOrName), LnxPortK, FALSE);
  Result := (Err = 0);
  ConnectionUp := Result;
END;



FUNCTION ReadALine (SockHandle : TSocket; VAR CurrLine : STRING) : BOOLEAN;
VAR C   : CHAR;
    Err : INTEGER;
BEGIN
  CurrLine := '';
  REPEAT
    Err := recv (SockHandle, C, 1, 0);
    IF Err <> 1 THEN
    BEGIN
      Result := FALSE;
      EXIT;
    END;
    IF (C = #13) OR (C = #10) THEN  // kommt da CR oder LF ?, egal
    BEGIN
      Result := TRUE;
      EXIT;
    END;
    IF C = #9 THEN C := ' ';
    CurrLine := CurrLine + C;
  UNTIL FALSE;
END;


PROCEDURE TIsdnInfoThread.Execute;
VAR CurrLine : STRING;
    S        : STRING;
    I        : INTEGER;



  FUNCTION NextWord : STRING;
  BEGIN
    Result := '';
    WHILE Copy (CurrLine,1,1) = ' ' DO DELETE (CurrLine,1,1);
    WHILE (Length (CurrLine) > 0) AND (Copy (CurrLine,1,1) <> ' ') DO
    BEGIN
      Result := Result + Copy (CurrLine,1,1);
      DELETE (CurrLine,1,1);
    END;
  END;

BEGIN
  // Send Reconnect-Message
  SendMessage (SignalWinHandle,
               IsdnMessageK,MsgParam_ConnectedK,
               LONGINT (@LnxIpAddrOrName));

  WHILE NOT Terminated DO
  BEGIN
    IF ReadALine (SockHandle, CurrLine) THEN
    BEGIN
      S := NextWord;
      I := 0;
      IF S = 'idmap:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          IsdnStatus [I].IdString := S;
          INC (I);
          S := NextWord;
        END;
      END ELSE
      IF S = 'chmap:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          IsdnStatus [I].ChannelLowLevelDriver := StrToInt(S);
          INC (I);
          S := NextWord;
        END;
      END ELSE
      IF S = 'drmap:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          IsdnStatus [I].IndexLowLevelDriver := StrToInt(S);
          INC (I);
          S := NextWord;
        END;
      END ELSE
      IF S = 'usage:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          IsdnStatus [I].Usage := StrToInt(S);
          INC (I);
          S := NextWord;
        END;
      END ELSE
      IF S = 'flags:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          DriverFlags[I] := PChar (S)^;
          INC (I);
          S := NextWord;
        END;
      END ELSE
      IF S = 'phone:' THEN
      BEGIN
        S := NextWord;
        WHILE S <> '' DO
        BEGIN
          IsdnStatus [I].Phone := S;
          INC (I);
          S := NextWord;
        END;
        // Phone ist die letzte Zeile -> Message an Hauptfenster senden
        SendMessage (SignalWinHandle,
                     IsdnMessageK,MsgParam_StatusChangeK,0);
      END
    END ELSE
    BEGIN
      IF SockHandle <> -1 THEN
      BEGIN
        // Send Disconnect-Message
        SendMessage (SignalWinHandle,
                       IsdnMessageK,MsgParam_DisconnectedK,0);
        closesocket (SockHandle);
        ConnectionUp := FALSE;
      END;
      Sleep (15000);  // dont stress the system...
      IF Connect THEN
      BEGIN
        ConnectionUp := TRUE;
        // Send Reconnect-Message
        SendMessage (SignalWinHandle,
                       IsdnMessageK,MsgParam_ConnectedK,LONGINT (@LnxIpAddrOrName));
      END;

    END;
  END;
END;




VAR i4ld_SockHandle : TSocket = 0;
    i4ld_Version    : STRING = '';

FUNCTION i4ld_Command (Cmd : STRING) : STRING;
BEGIN
  Cmd := Cmd + #13;
  Result := '!ERROR';
  IF send (i4ld_SockHandle,PChar (Cmd)^, Length (Cmd), 0) <> Length (Cmd) THEN
  BEGIN
    closeSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    EXIT;
  END;
  IF NOT ReadALine (i4ld_SockHandle, Result) THEN
  BEGIN
    closeSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    Result := '!ERROR';
  END;
END;





// Returns List of ippp interfaces separated by space
FUNCTION i4ld_GetIppps   (VAR   IpppList : STRING) : BOOLEAN;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  IpppList := i4ld_Command ('list if');
  Result := Copy (IpppList,1,2) = 'Ok';
  IF Result THEN Delete (IpppList,1,3);
END;

// dials interface IpppName
FUNCTION i4ld_Dial       (CONST IpppName : STRING) : BOOLEAN;
VAR S : STRING;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  IF IpppName = '' THEN
  BEGIN
    Result := FALSE;
    EXIT;
  END;

  S := i4ld_Command ('dial '+IpppName);
  Result := Copy (S,1,2) = 'Ok';
END;

// hangup interface IpppName
FUNCTION i4ld_Hangup     (CONST IpppName : STRING) : BOOLEAN;
VAR S : STRING;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  IF IpppName = '' THEN
  BEGIN
    Result := FALSE;
    EXIT;
  END;

  S := i4ld_Command ('hangup '+IpppName);
  Result := Copy (S,1,2) = 'Ok';
END;

// connect via tcp to i4lmond
FUNCTION i4ld_Connect    (LinuxAddressOrName     : STRING) : BOOLEAN;
VAR Err : INTEGER;
    S   : STRING;
BEGIN
  Result := i4ld_Connected;
  IF Result THEN EXIT;

  Err := TCPConnectTo (i4ld_SockHandle,PChar(LinuxAddressOrName),
                       LnxPortK+1,FALSE);
  IF Err <> 0 THEN
  BEGIN
    i4ld_SockHandle := 0;
    Result := FALSE;
    EXIT;
  END;
  // wait for READY
  IF NOT ReadALine (i4ld_SockHandle, S) THEN
  BEGIN
    CloseSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    Result := FALSE;
    EXIT;
  END;

  IF Copy (S,1,5) <> 'READY' THEN
  BEGIN
    CloseSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    Result := FALSE;
    EXIT;
  END;
  i4ld_Version := Copy (S,7,255);
  Result := TRUE;
END;

// disconnect from i4lmond
FUNCTION i4ld_DisConnect : BOOLEAN;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  // Damit der i4lmond beenden wird, schicken wir ein Quit
  i4ld_Command ('QUIT 0');
  closeSocket (i4ld_SockHandle);
  i4ld_SockHandle := 0;
  Result := TRUE;
END;

// are we connected to i4lmond ?
FUNCTION i4ld_Connected : BOOLEAN;
BEGIN
  Result := (i4ld_SockHandle <> 0);
END;


// Fill info into StGrid
FUNCTION i4ld_ListIppp (CONST IpppName : STRING;
                              StGrid   : TStringGrid) : BOOLEAN;
VAR St,St1 : STRING;
    P      : INTEGER;
    L      : INTEGER;
BEGIN
  StGrid.RowCount := 0;
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  StGrid.ColWidths[0] := 180;

  Result := FALSE;
  St1 := i4ld_Command ('list '+IpppName);
  L := 0;
  WHILE (Copy (St1,1,6) <> '!ERROR') AND (Copy (St1,1,2) <> 'Ok') DO
  BEGIN
    St := St1;
    IF St <> '' THEN
    BEGIN
      Result := TRUE;
      P := Pos (':', St);
      StGrid.RowCount := StGrid.RowCount + 1;
      IF P = 0 THEN
      BEGIN
        StGrid.Cells [0,L] := St;
        StGrid.Cells [1,L] := '';
      END ELSE
      BEGIN
        StGrid.Cells [0,L] := Copy (St,1,P-1);
        Delete (St,1,P); St := Trim(St);
        StGrid.Cells [1,L] := St;
      END;
      INC (L);
    END;
    IF NOT ReadALine (i4ld_SockHandle, St1) THEN
      St1 := '!ERROR';
  END;
END;


FUNCTION i4ld_GetOutoingNumbers (CONST IpppName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IpppName = '' THEN
    EXIT;

  Result := i4ld_Command ('numout '+IpppName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION i4ld_GetPhonbookEntry (CONST Number : STRING) : STRING;
BEGIN
  Result := Number;
  IF NOT i4ld_Connected THEN EXIT;

  IF Number = '' THEN
    EXIT;

  Result := i4ld_Command ('getphonebook '+Number);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;    


INITIALIZATION
  FillChar (IsdnStatus, SIZEOF (IsdnStatus), 0);
END.
