//  $HDR$
// **********************************************************************
//  Armin Diehl (diehl@nordrhein.de)
// **********************************************************************
//  $Log:  \\FS-AD\PRJ\GP-Version\Archives\i4l Monitor\I4lmon\Compon\Isdn4Linux.pas.z 
// 
//    Rev 1.1    19.01.2001 21:22:59  ad
//  wrong case for quit-command (thanks to Stefan Dringenberg 
//  <dringenberg@gmx.net>)

// 
//    Rev 1.0    24.12.2000 15:22:29  ad
//  Initial Revision


// 
UNIT Isdn4Linux;

INTERFACE

// AD 15.11.1998: in TIsdn4Linux.i4ld_Command Abschlu mit #10 anstatt #13
//                da fgets des neuen i4lmond mit #13 nicht luft
// AD 24.12.1999: cat /dev/isdninfo ersetzt durch tail -f -n1 | /sbin/isdninfo
//                das erste Zeichen jeder Zeile ist jetzt '0' fr /dev/isdninfo
//                und '1' fr Infos aus /var/log/messages
// AD 06.06.2000: Reconnect now working, realy ;-)

USES
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WinSock, Grids;

CONST I4LMaxChannelsK  = 64;
      I4LMaxCardsK     = 16;
      I4LMaxInterfacesK= 64;
      IsdnMessageK     = WM_USER + 1;
      MsgParam_StatusChangeK = 0;
      MsgParam_DisconnectedK = 1;
      MsgParam_ConnectedK    = 2;  // only after a disconnect, Data in IsdnStatus may not be valid
      I4LDefPortK            = 6105;
      i4ld_NotConnectedK     = 'Keine Verbindung zu i4lmond';
      i4ld_NoInterfaceK      = 'Kein Interfacename angegeben';


TYPE
  Ti4lService = (IS_NoneK,
                 IS_RawK,
                 IS_ModemK,
                 IS_NetK,
                 IS_VoiceK);

  Str10 = STRING [10];

CONST ServiceTxtK : ARRAY [IS_NoneK..IS_VoiceK] OF Str10 =
  ('Frei','Raw','Modem','Net','Voice');

TYPE
  TInterfaceConnectStatus = (IFSUnknownK, IFSNotConnectedK, IFSConnectedInK, IFSConnectedOutK);
  TIsdnStatus = RECORD
                  Index         : BYTE;
                  IdString      : STRING [30];      // i.e. ISDN0
                  PhoneNumber   : STRING [30];
                  PhonebookName : STRING [30];      // no Entry -> PhoneNumber
                  Service       : Ti4lService;
                  Outgoing      : BOOLEAN;
                  Exclusive     : BOOLEAN;
                  DriverNum     : SHORTINT;    // used Isdn Card (-1 = none)
                  BChannelNum   : BYTE;        // Number for Driver
                  BChannelActive: BOOLEAN;     // BChannel Used
                  Time          : TDateTime;
                END;
  PIsdnStatus = ^TIsdnStatus;

  Ti4lStatusEvent   = PROCEDURE (    Sender               : TObject;
                                 VAR NewStatus, OldStatus : TIsdnStatus) OF OBJECT;

  Ti4lActionEvent   = PROCEDURE (    Sender               : TObject;
                                 VAR NewStatus            : TIsdnStatus) OF OBJECT;

  Ti4lBChannelEvent = PROCEDURE (Sender       : TObject;
                                 DriverLoaded : BOOLEAN;
                                 DriverNum    : BYTE;
                                 B1,B2        : BYTE) OF OBJECT;

  // incoming Call (from /var/log/messages)
  Ti4lUnboundCallEvent = PROCEDURE (Sender       : TObject;
                                    CallFrom,
                                    CallTo       : STRING;
                                    Service      : Ti4lService) OF OBJECT;



  TIsdnInfoThread = CLASS;

  TIsdn4LinuxException = CLASS (Exception);
  TIsdn4Linux = class(TComponent)
  PRIVATE
    { Private-Deklarationen }
    FLinuxAddrOrName      : STRING;
    FLinuxPort            : WORD;
    FOnLinuxConnect       : TNotifyEvent;
    FOnLinuxDisconnect    : TNotifyEvent;
    FOnStatusChange       : Ti4lStatusEvent;
    FOnDial               : Ti4lActionEvent;
    FOnRing               : Ti4lActionEvent;
    FOnConnect            : Ti4lActionEvent;
    FOnDisconnect         : Ti4lStatusEvent;
    FOnBChannelChange     : Ti4lBChannelEvent;
    FOnTerminate          : Ti4lStatusEvent;
    FUnboundCallEvent     : Ti4lUnboundCallEvent;
    MSG_UnableToConnect   : STRING;
    
  PUBLIC
    IsdnInfoThread        : TIsdnInfoThread;
  PRIVATE
    i4ld_SockHandle       : TSocket;
    fi4ld_Version         : STRING;

    FNumIsdnInterfaces    : INTEGER;
    FNumCBIsdnInterfaces  : INTEGER;
    // Interfaces
    FIsdnInterfaceNames   : ARRAY [0..I4LMaxInterfacesK-1] OF STRING;
    // Interfaces with Channel-Bundling
    FcbIsdnInterfaceNames : ARRAY [0..I4LMaxInterfacesK-1] OF STRING;

    PROCEDURE ClearStatus (VAR X : ARRAY OF TIsdnStatus);

    FUNCTION  GetConnected : BOOLEAN;
    FUNCTION  GetInterfaceName (Index : INTEGER) : STRING;
    FUNCTION  GetCBInterfaceName (Index : INTEGER) : STRING;
    FUNCTION  GetInterfaceCount : INTEGER;
    FUNCTION  GetCBInterfaceCount : INTEGER;
    FUNCTION  GetInterfaceOutNumbers (Index : INTEGER) : STRING;
    FUNCTION  GetInterfaceInNumbers (Index : INTEGER) : STRING;
    FUNCTION  GetInterfaceDialmode (Index : INTEGER) : STRING;
    PROCEDURE SetInterfaceDialmode (Index : INTEGER; Dialmode : STRING);
  PROTECTED
    { Protected-Deklarationen }
    PROCEDURE SetLinuxAddrOrName (S : STRING);
    PROCEDURE SetLinuxPort       (P : WORD);
  PUBLIC
    foldStatus            : ARRAY [0..I4LMaxChannelsK-1] OF TIsdnStatus;
    fnewStatus            : ARRAY [0..I4LMaxChannelsK-1] OF TIsdnStatus;
    foldBChannelUsage     : ARRAY [0..I4LMaxCardsK-1] OF BYTE;
    fnewBChannelUsage     : ARRAY [0..I4LMaxCardsK-1] OF BYTE;
    fBChannelOutgoing     : ARRAY [0..I4LMaxCardsK-1] OF BYTE;
    fnewDriverLoaded      : ARRAY [0..I4LMaxCardsK-1] OF BOOLEAN;
    foldDriverLoaded      : ARRAY [0..I4LMaxCardsK-1] OF BOOLEAN;

    CONSTRUCTOR Create (aOwner : TComponent); OVERRIDE;
    DESTRUCTOR Destroy; OVERRIDE;

    { Public-Deklarationen }
    // Calld by the Thread on Status Changed
    PROCEDURE IsdnInfoThreadEvent (Reason    : BYTE);

    // Called by the Thread on a isdn_tty-Message from /var/log/messages
    PROCEDURE IsdnUnboundCallEvent (CallFrom,
                                    CallTo       : STRING;
                                    Service      : Ti4lService);


    // communication with i4lmond
    FUNCTION i4ld_Command (Cmd : STRING) : STRING;

    // dials interface IpppName
    PROCEDURE Dial       (CONST IfName : STRING);

    // hangup interface IpppName
    PROCEDURE Hangup     (CONST IfName : STRING);

    // connect via tcp to i4lmond
    FUNCTION i4ld_Connect : BOOLEAN;

    // disconnect from i4lmond
    FUNCTION i4ld_DisConnect : BOOLEAN;

    // are we connected to i4lmond ?
    FUNCTION i4ld_Connected : BOOLEAN;

    // Test Connection via Version-Command
    FUNCTION i4ld_TestConnected : BOOLEAN;

    // Fill Isdn-info (=isdnctrl list) into StGrid
    FUNCTION i4ld_ListIf (CONST IfName : STRING;
                                StGrid   : TStringGrid) : BOOLEAN;

    PROCEDURE MarkAllDriverUnloaded;

    FUNCTION i4ld_GetOutgoingNumbers (CONST IfName : STRING) : STRING;
    FUNCTION i4ld_GetIncomingNumbers (CONST IfName : STRING) : STRING;
    FUNCTION i4ld_GetInterfaceConnectionStatus
                                     (CONST IfName : STRING) : TInterfaceConnectStatus;
    FUNCTION i4ld_GetDialmode        (CONST IfName : STRING) : STRING;
    FUNCTION i4ld_SetDialmode        (CONST IfName, Dialmode : STRING) : STRING;
    FUNCTION i4ld_DelNumber          (CONST IfName, Number : STRING; Outgoing : BOOLEAN) : STRING;
    FUNCTION i4ld_AddNumber          (CONST IfName, Number : STRING; Outgoing : BOOLEAN) : STRING;
    FUNCTION i4ld_AddLink            (CONST IfName : STRING) : STRING;
    FUNCTION i4ld_RemoveLink         (CONST IfName : STRING) : STRING;


    FUNCTION GetPhonebookEntry (Number : STRING) : STRING;
    FUNCTION GetInterfaceNumber (Name : STRING) : INTEGER;

    // Refresh List of Interfaces
    PROCEDURE i4ld_GetInterfaces;

    PROCEDURE SetErrorText (_MSG_UnableToConnect : STRING);


  PUBLISHED
    { Published-Deklarationen }
    PROPERTY LinuxAddrOrName  : STRING READ FLinuxAddrOrName WRITE SetLinuxAddrOrName;
    PROPERTY LinuxPort        : WORD   READ FLinuxPort       WRITE SetLinuxPort;

    PROPERTY OnLinuxConnect   : TNotifyEvent    READ fOnLinuxConnect    WRITE fOnLinuxConnect;
    PROPERTY OnLinuxDisconnect: TNotifyEvent    READ fOnLinuxDisconnect WRITE fOnLinuxDisconnect;

    PROPERTY OnStatusChange   : Ti4lStatusEvent READ fOnStatusChange    WRITE fOnStatusChange;
    PROPERTY OnDial           : Ti4lActionEvent READ fOnDial            WRITE fOnDial;
    PROPERTY OnRing           : Ti4lActionEvent READ fOnRing            WRITE fOnRing;
    PROPERTY OnConnect        : Ti4lActionEvent READ fOnConnect         WRITE fOnConnect;
    PROPERTY OnDisconnect     : Ti4lStatusEvent READ fOnDisconnect      WRITE fOnDisconnect;
    PROPERTY OnBChannelChange : Ti4lBChannelEvent READ fOnBChannelChange WRITE fOnBChannelChange;
    PROPERTY OnTerminate      : Ti4lStatusEvent   READ FOnTerminate      WRITE FOnTerminate;
    PROPERTY i4ldVersion      : STRING            READ fi4ld_Version;
    PROPERTY i4ldConnected    : BOOLEAN           READ i4ld_Connected;
    PROPERTY i4ldTestConnected: BOOLEAN           READ i4ld_TestConnected;
    PROPERTY InterfaceCount   : INTEGER           READ GetInterfaceCount;
    PROPERTY CBInterfaceCount : INTEGER           READ GetCBInterfaceCount;
  PUBLIC
    // Interfaces
    PROPERTY InterfaceNames [Index: Integer]: STRING READ GetInterfaceName;
    // Interfaces with Channel Bundling
    PROPERTY CBInterfaceNames [Index: Integer]: STRING READ GetCBInterfaceName;
    PROPERTY InterfaceOutNumbers [Index : INTEGER] : STRING READ GetInterfaceOutNumbers;
    PROPERTY InterfaceInNumbers [Index : INTEGER] : STRING READ GetInterfaceInNumbers;
    PROPERTY InterfaceDialmode  [Index : INTEGER] : STRING READ GetInterfaceDialmode WRITE SetInterfaceDialmode;
    PROPERTY Phonebook [Number : STRING] : STRING READ GetPhonebookEntry;
    PROPERTY InterfaceNumber [Name : STRING] : INTEGER READ GetInterfaceNumber;
  PUBLISHED  
    PROPERTY Connected        : BOOLEAN READ GetConnected;
    PROPERTY OnUnboundCall    : Ti4lUnboundCallEvent READ FUnboundCallEvent WRITE FUnboundCallEvent;
  END;


  TIsdnInfoThread = CLASS (TTHread)
    SockHandle      : TSocket;
    SignalWinHandle : HWND;
    LnxIpAddrOrName : STRING;
    Owner           : TIsdn4Linux;
    Reason          : BYTE;
    UBCallFrom,
    UBCallTo        : STRING;
    UBService       : INTEGER;


    PROCEDURE CallIsdnInfoThreadEvent;
    PROCEDURE CallUnboundCallThreadEvent;

    CONSTRUCTOR Create (LinuxAddressOrName     : STRING;
                        aOwner                 : TIsdn4Linux);
    DESTRUCTOR Destroy; OVERRIDE;
    PROCEDURE Execute; OVERRIDE;
    FUNCTION  Connect : BOOLEAN;
    PROCEDURE Terminate;
  END;

PROCEDURE Register;

CONST TreadEvent_LinuxConnectK    = 1;
      TreadEvent_LinuxDisconnectK = 2;
      TreadEvent_StatusChangeK    = 3;
      TreadEvent_StatusChangeAllK = 4;
      LastLinuxResponse : STRING = '';


IMPLEMENTATION


PROCEDURE InitWinSock;
VAR WSData: TWSAData;
BEGIN
  WSAStartup ($0101,WSData);
END;



FUNCTION TCPConnectTo (VAR aSocket      : TSocket;
                           AdressOrName : PCHAR;
                           Port         : WORD;
                           SetNDelay    : BOOLEAN) : INTEGER;
VAR Remote_Addr : TSockAddrIn;
    Remote_Host : Phostent;
    A           : TInAddr;
    OptVal      : INTEGER;
BEGIN
  Remote_Addr.sin_addr.S_addr := 0;
  // ersteinmal versuchen in eine Adresse umzuwandeln
  u_long (A) := inet_addr (AdressOrName);
  IF longword (A) = INADDR_NONE THEN
  BEGIN
    // ist eventuell ein Hostname
    Remote_Host := gethostbyname(AdressOrName);
    IF Remote_Host <> NIL THEN
      Remote_Addr.sin_addr := PInAddr(Remote_Host^.h_addr_list^)^;
  END ELSE
    Remote_Addr.sin_addr.S_addr := u_long (A);

  Remote_Addr.sin_family := PF_INET;
  Remote_Addr.sin_port := htons (Port);

  // Ok, Host bekannt, socket holen
  IF Remote_Addr.sin_addr.S_addr = 0 THEN
  BEGIN
    Result := WSAGetLastError;
    IF Result = 0 THEN Result := 1;
    aSocket := INVALID_SOCKET;
    EXIT;
  END ELSE
    aSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);

  IF aSocket = INVALID_SOCKET THEN
  BEGIN
    Result := WSAGetLastError;
    IF Result = 0 THEN INC (Result);
    EXIT;
  END;

  IF connect (aSocket, Remote_Addr, SIZEOF (Remote_Addr)) <> 0 THEN
  BEGIN
    closesocket (aSocket);
    Result := WSAGetLastError;
    aSocket := INVALID_SOCKET;
    IF Result = 0 THEN INC (Result);
    EXIT;
  END;

  IF SetNDelay THEN
  BEGIN
    OptVal := 1;
    setsockopt (aSocket, IPPROTO_TCP, TCP_NODELAY, @OptVal, SIZEOF(OptVal));
  END;
  Result := 0;
END;


PROCEDURE TIsdnInfoThread.CallIsdnInfoThreadEvent;
BEGIN
  IF Owner <> NIL THEN
    Owner.IsdnInfoThreadEvent (Reason);
END;

PROCEDURE TIsdnInfoThread.CallUnboundCallThreadEvent;
BEGIN
  IF Owner <> NIL THEN
    Owner.IsdnUnboundCallEvent (UBCallFrom, UBCallTo, TI4lService(UBService));
END;


CONSTRUCTOR TIsdnInfoThread.Create (LinuxAddressOrName     : STRING;
                                    aOwner                 : TIsdn4Linux);
VAR Err : INTEGER;
BEGIN
  INHERITED Create (TRUE);  // luft noch nicht
  FreeOnTerminate := TRUE;
  SockHandle := -1;
  Owner := aOwner;
  LnxIpAddrOrName := LinuxAddressOrName;
  Err := TCPConnectTo (SockHandle, PChar(LnxIpAddrOrName), aOwner.LinuxPort, TRUE);
  IF Err = WSANOTINITIALISED THEN
  BEGIN
    InitWinSock;
    Err := TCPConnectTo (SockHandle, PChar(LnxIpAddrOrName), aOwner.LinuxPort, TRUE);
  END;
  IF Err <> 0 THEN
    //RAISE (TIsdn4LinuxException.Create ('Verbindung zu '+LinuxAddressOrName+' Port '+IntToStr(aOwner.LinuxPort)+' konnte leider nicht hergestellt werden ('+IntToStr(Err)+')'));
    RAISE (TIsdn4LinuxException.CreateFmt (aOwner.MSG_UnableToConnect,[LinuxAddressOrName,aOwner.LinuxPort,Err]));

  Suspended := FALSE;
  Reason := TreadEvent_LinuxConnectK;
  Synchronize (CallIsdnInfoThreadEvent);
END;

DESTRUCTOR TIsdnInfoThread.Destroy;
BEGIN
  IF SockHandle <> -1 THEN
  BEGIN
    closesocket (SockHandle);
    Reason := TreadEvent_LinuxDisconnectK;
    Synchronize (CallIsdnInfoThreadEvent);
  END;
  IF Owner <> NIL THEN
    Owner.IsdnInfoThread := NIL;
  INHERITED Destroy;
END;



FUNCTION  TIsdnInfoThread.Connect : BOOLEAN;
VAR Err : INTEGER;
BEGIN
  Err := TCPConnectTo (SockHandle, PChar(LnxIpAddrOrName), Owner.LinuxPort, TRUE);
  Result := (Err = 0);
END;



FUNCTION ReadALine (VAR SockHandle : TSocket; VAR CurrLine : STRING) : BOOLEAN;
VAR C      : CHAR;
    Err    : INTEGER;
BEGIN
  CurrLine := '';
  LastLinuxResponse := '';
  REPEAT
    Err := recv (SockHandle, C, 1, 0);
    IF Err <> 1 THEN
    BEGIN
      closeSocket (SockHandle);
      SockHandle := -1;
      Result := FALSE;
      EXIT;
    END;
    IF (C = #13) OR (C = #10) THEN  // kommt da CR oder LF ?, egal
    BEGIN
      LastLinuxResponse := CurrLine;
      Result := TRUE;
      EXIT;
    END;
    IF C = #9 THEN C := ' ';
    CurrLine := CurrLine + C
  UNTIL FALSE;
END;


PROCEDURE TIsdnInfoThread.Execute;
VAR CurrLine    : STRING;
    S           : STRING;
    I,J,Err     : INTEGER;

  FUNCTION NextWord : STRING;
  BEGIN
    Result := '';
    WHILE Copy (CurrLine,1,1) = ' ' DO DELETE (CurrLine,1,1);
    WHILE (Length (CurrLine) > 0) AND (Copy (CurrLine,1,1) <> ' ') DO
    BEGIN
      Result := Result + Copy (CurrLine,1,1);
      DELETE (CurrLine,1,1);
    END;
  END;

BEGIN
  WHILE NOT Terminated DO
  BEGIN
    IF ReadALine (SockHandle, CurrLine) THEN
    BEGIN
      IF Owner <> NIL THEN
      BEGIN
        IF PChar(CurrLine)^ = '0' THEN  // line from /dev/isdninfo
        BEGIN
          Delete (CurrLine,1,1);
          S := NextWord;
          I := 0;
          IF S = 'idmap:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxChannelsK) DO
            BEGIN
              IF S = '-' THEN S := '';
              Owner.fnewStatus [I].IdString := S;
              INC (I);
              S := NextWord;
            END;
          END ELSE
          IF S = 'chmap:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxChannelsK) DO
            BEGIN
              J := StrToInt(S);
              IF J < 0 THEN J := 0;
              Owner.fnewStatus [I].BChannelNum := J;
              INC (I);
              S := NextWord;
            END;
          END ELSE
          IF S = 'drmap:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxChannelsK) DO
            BEGIN
              Owner.fnewStatus [I].DriverNum := StrToInt(S);
              INC (I);
              S := NextWord;
            END;
          END ELSE
          IF S = 'usage:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxChannelsK) DO
            BEGIN
              J := StrToInt(S);
              BYTE (Owner.fnewStatus [I].Service) := J AND $F;
              Owner.fnewStatus [I].Exclusive := (J AND 64 = 64);
              Owner.fnewStatus [I].Outgoing := J AND 128 <> 0;
              INC (I);
              S := NextWord;
            END;
          END ELSE
          IF S = 'flags:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxCardsK) DO
            BEGIN
              IF S <> '?' THEN
              BEGIN
                Owner.fnewBChannelUsage[I] := StrToInt (S);
                Owner.fnewDriverLoaded[I] := TRUE;
              END ELSE
              BEGIN
                Owner.fnewBChannelUsage[I] := 0;
                Owner.fnewDriverLoaded[I] := FALSE;
              END;
              INC (I);
              S := NextWord;
            END;
          END ELSE
          IF S = 'phone:' THEN
          BEGIN
            S := NextWord;
            WHILE (S <> '') AND (I < I4LMaxChannelsK) DO
            BEGIN
              IF S = '???' THEN S := '';
              IF S = '' THEN
                Owner.fnewStatus[I].PhonebookName := ''
              ELSE
              IF (S <> Owner.fnewStatus[I].PhoneNumber) THEN
                Owner.fnewStatus[I].PhonebookName := Owner.GetPhonebookEntry (S);
              Owner.fnewStatus [I].PhoneNumber := S;
              INC (I);
              S := NextWord;
            END;

            // set BChannelActive
            FOR I := 0 TO I4LMaxChannelsK-1 DO
            WITH Owner.fnewStatus [I] DO
            BEGIN
              BChannelActive := FALSE;
              IF DriverNum > -1 THEN
              BEGIN
                BChannelActive := ((Owner.fnewBChannelUsage[DriverNum] SHL BChannelNum) AND 1 = 1);
                IF BChannelActive AND Outgoing THEN
                  Owner.fBChannelOutgoing [DriverNum] := Owner.fBChannelOutgoing [DriverNum] OR (1 SHL BChannelNum)
                ELSE
                  Owner.fBChannelOutgoing [DriverNum] := Owner.fBChannelOutgoing [DriverNum] AND (NOT (1 SHL BChannelNum));
              END;
            END;

            // Phone ist die letzte Zeile -> Message an Hauptfenster senden
            Reason := TreadEvent_StatusChangeK;
            Synchronize (CallIsdnInfoThreadEvent);
          END
        END ELSE
        IF PChar(CurrLine)^ = '1' THEN  // Line from /var/log/messages
        BEGIN
          Delete (CurrLine,1,1);
          UBCallFrom := NextWord;
          UBCallTo   := NextWord;
          S        := NextWord;
          Val (S, UBService, Err);
          IF Err <> 0 THEN UBService := 0;
          IF UBService = 0 THEN
            UBService := BYTE(IS_VoiceK);
          // we do not need time and date
          Synchronize (CallUnboundCallThreadEvent);

        END;

      END
    END ELSE
    BEGIN
      IF SockHandle = -1 THEN
        IF Owner <> NIL THEN
        BEGIN
          // Send Disconnect-Message
          closesocket (SockHandle);
          Reason := TreadEvent_LinuxDisconnectK;
          Synchronize (CallIsdnInfoThreadEvent);
          SockHandle := -1;
          Owner.MarkAllDriverUnloaded;
        END;
      Sleep (100);
      IF Terminated THEN
      BEGIN
        Reason := TreadEvent_LinuxDisconnectK;
        Synchronize (CallIsdnInfoThreadEvent);
        EXIT;
      END;
      Sleep (15000);  // dont stress the system...
      IF Connect THEN
      BEGIN
        // Send Reconnect-Message
        Reason := TreadEvent_LinuxConnectK;
        Synchronize (CallIsdnInfoThreadEvent);
      END;

    END;
  END;
END;


PROCEDURE TIsdnInfoThread.Terminate;
BEGIN
  IF SockHandle <> -1 THEN
  BEGIN
    closesocket (SockHandle);
    SockHandle := -1;
  END;
  INHERITED Terminate;
END;


(****************************************************************************)

CONSTRUCTOR TIsdn4Linux.Create (aOwner : TComponent);
BEGIN
  INHERITED Create (aOwner);
  fLinuxPort := I4LDefPortK;
  i4ld_SockHandle := -1;
  IsdnInfoThread := NIL;
  MSG_UnableToConnect := 'Verbindung zu %s Port %d konnte leider nicht hergestellt werden (%d)';
END;


DESTRUCTOR TIsdn4Linux.Destroy;
BEGIN
  IF IsdnInfoThread <> NIL THEN
  BEGIN
    IsdnInfoThread.Owner := NIL;
    IsdnInfoThread.Terminate;
    IsdnInfoThread := NIL;
  END;
  INHERITED Destroy;
END;


FUNCTION  TIsdn4Linux.GetConnected : BOOLEAN;
BEGIN
  Result := (IsdnInfoThread <> NIL);
  IF Result THEN
    Result := (IsdnInfoThread.SockHandle <> -1);
END;

PROCEDURE TIsdn4Linux.ClearStatus (VAR X : ARRAY OF TIsdnStatus);
VAR I : INTEGER;
BEGIN
  FillChar (X, SIZEOF (X), 0);
  FOR I := LOW (X) TO HIGH (X) DO
  BEGIN
    X[I].DriverNum := -1;
    X[I].Index := I;
  END;
END;


PROCEDURE TIsdn4Linux.SetLinuxAddrOrName (S : STRING);
VAR I : INTEGER;
BEGIN
  IF IsdnInfoThread <> NIL THEN
  BEGIN
    IsdnInfoThread.Terminate;
    I := 0;
    WHILE (I < 20) AND (IsdnInfoThread <> NIL) DO
    BEGIN
      Sleep (100);
      INC (I);
    END;
  END;
  i4ld_Disconnect;

  FLinuxAddrOrName := S;
  ClearStatus (foldStatus);
  ClearStatus (fnewStatus);
  FillChar (fnewBChannelUsage, SIZEOF (fnewBChannelUsage), 0);
  FillChar (foldBChannelUsage, SIZEOF (foldBChannelUsage), 0);
  FillChar (fBChannelOutgoing, SIZEOF (fBChannelOutgoing), 0);
  FillChar (fnewDriverLoaded, SIZEOF (fnewDriverLoaded), 0);
  FillChar (foldDriverLoaded, SIZEOF (foldDriverLoaded), 0);
  IF S <> '' THEN
  BEGIN
    i4ld_Connect;
    IsdnInfoThread := TIsdnInfoThread.Create (S, SELF);
  END;
END;


PROCEDURE TIsdn4Linux.SetLinuxPort (P : WORD);
BEGIN
  IF P <> FLinuxPort THEN
  BEGIN
    fLinuxPort := P;
    SetLinuxAddrOrName (FLinuxAddrOrName);
  END;
END;



FUNCTION CompareStatus (CONST Old,New : TIsdnStatus) : BOOLEAN;
BEGIN
  Result := FALSE;
  IF Old.DriverNum <> New.DriverNum THEN EXIT;
  IF New.DriverNum = -1 THEN
  BEGIN
    Result := TRUE;
    EXIT;
  END;
  IF Old.BChannelActive <> New.BChannelActive THEN EXIT;
  IF Old.IdString <> New.IdString THEN EXIT;
  IF Old.PhoneNumber <> New.PhoneNumber THEN EXIT;
  //PhonebookName : STRING;      // no Entry -> PhoneNumber
  IF Old.Service <> New.Service THEN EXIT;
  IF Old.Outgoing  <> New.Outgoing THEN EXIT;
  IF Old.Exclusive <> New.Exclusive THEN EXIT;
  IF Old.BChannelNum <> New.BChannelNum THEN EXIT;
  Result := TRUE;
END;

// Calld by the Thread on Status Changed
PROCEDURE TIsdn4Linux.IsdnInfoThreadEvent (Reason    : BYTE);
VAR I : INTEGER;
    B1,B2 : BYTE;
BEGIN
  CASE Reason OF
    TreadEvent_LinuxConnectK    :
      IF Assigned (FOnLinuxConnect) THEN FOnLinuxConnect (SELF);
    TreadEvent_LinuxDisconnectK :
      BEGIN
        IF Assigned (FOnLinuxDisconnect) THEN FOnLinuxDisconnect (SELF);
        ClearStatus (fnewStatus);
        IsdnInfoThreadEvent (TreadEvent_StatusChangeK);
      END;
    TreadEvent_StatusChangeK,
    TreadEvent_StatusChangeAllK :
    BEGIN
      FOR I := 0 TO I4LMaxChannelsK-1 DO
      BEGIN
        IF (Reason = TreadEvent_StatusChangeAllK) OR
        (NOT CompareStatus (foldStatus[I],fnewStatus[I])) THEN
        BEGIN
          fnewStatus [I].Time := Sysutils.now;
          // Changed
          IF Assigned (FOnStatusChange) THEN
            FOnStatusChange (SELF, fnewStatus[I],foldStatus[I]);

          // on Dial
          // Outgoing Status-Flag currently (03.10.1998) not correct
          // in i4l. Outgoing is false on dial so this will currently
          // not work.
          IF Assigned (FOnDial) THEN
            IF fnewStatus[I].Service <> IS_NoneK THEN
              IF fnewStatus[I].Service <> foldStatus[I].Service THEN
                IF NOT fnewStatus[I].BChannelActive THEN  // not Connected
                  IF fnewStatus[I].Outgoing THEN          // and Outgoing
                    fOnDial (SELF,fnewStatus[I]);

          // on Ring
          IF Assigned (FOnRing) THEN
            IF fnewStatus[I].Service <> IS_NoneK THEN
              IF fnewStatus[I].Service <> foldStatus[I].Service THEN
                IF NOT fnewStatus[I].BChannelActive THEN  // not Connected
                  IF NOT fnewStatus[I].Outgoing THEN      // and incoming
                    fOnRing (SELF,fnewStatus[I]);

          // on Connect
          IF Assigned (FOnConnect) THEN
            // B-Channel changed ?
            IF fnewStatus[I].BChannelActive <> foldStatus[I].BChannelActive THEN
              // and new Channel in use
              IF fnewStatus[I].BChannelActive THEN
                fOnConnect (SELF,fnewStatus[I]);

          // on Disconnect
          IF Assigned (FOnDisconnect) THEN
            // B-Channel Status changed ?
            IF fnewStatus[I].BChannelActive <> foldStatus[I].BChannelActive THEN
              // and new Channel is not active
              IF NOT fnewStatus[I].BChannelActive THEN
                fOnDisconnect (SELF,fnewStatus[I],foldStatus[I]);

          // on Terminate
          IF Assigned (FOnTerminate) THEN
            // B-Channel Status not changed ?
            IF fnewStatus[I].BChannelActive = foldStatus[I].BChannelActive THEN
              // B-Channel not active
              IF NOT fnewStatus[I].BChannelActive THEN
                // Status=Free
                IF fnewStatus[I].Service = IS_NoneK THEN
                  // old Status not Free
                  IF foldStatus[I].Service <> IS_NoneK THEN
                    fOnTerminate (SELF,fnewStatus[I],foldStatus[I]);

          foldStatus[I] := fnewStatus[I];
        END;
      END;
      // B-Channels
      IF Assigned (FOnBChannelChange) THEN
        FOR I := 0 TO I4LMaxCardsK-1 DO
        BEGIN
          IF (Reason = TreadEvent_StatusChangeAllK) OR (
             (fnewBChannelUsage[I] <> foldBChannelUsage[I]) OR
             (fnewDriverLoaded[I] <> foldDriverLoaded[I]) OR
             (fnewDriverLoaded[I] <> foldDriverLoaded[I])) THEN
          BEGIN
            B1 := fnewBChannelUsage[I] AND 1;
            B2 := fnewBChannelUsage[I] AND 2;
            IF (B1 > 0) AND (fBChannelOutgoing[I] AND 1 = 1) THEN INC (B1);
            IF (B2 > 0) AND (fBChannelOutgoing[I] AND 2 = 2) THEN INC (B2);
            FOnBChannelChange (SELF,fnewDriverLoaded[I],I,B1,B2);
            foldBChannelUsage[I] := fnewBChannelUsage[I];
            foldDriverLoaded[I] := fnewDriverLoaded[I];
          END;
        END;
    END;
  END;
END;


// Called by the Thread on a isdn_tty-Message from /var/log/messages
PROCEDURE TIsdn4Linux.IsdnUnboundCallEvent (CallFrom,
                                            CallTo       : STRING;
                                            Service      : Ti4lService);
BEGIN
  IF Assigned (FUnboundCallEvent) THEN
    FUnboundCallEvent (SELF, CallFrom, CallTo, Service);
END;




FUNCTION TIsdn4Linux.i4ld_Command (Cmd : STRING) : STRING;
BEGIN
  Cmd := Cmd + #10;
  Result := '!ERROR';
  IF send (i4ld_SockHandle,PChar (Cmd)^, Length (Cmd), 0) <> Length (Cmd) THEN
  BEGIN
    LastLinuxResponse := 'none (winsock.send failed)';
    closeSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    EXIT;
  END;
  IF NOT ReadALine (i4ld_SockHandle, Result) THEN
  BEGIN
    closeSocket (i4ld_SockHandle);
    i4ld_SockHandle := 0;
    Result := '!ERROR isdn4linux.ReadALine returned false';
  END;
END;



// Returns List of interfaces separated by space
PROCEDURE TIsdn4Linux.i4ld_GetInterfaces;
VAR S : STRING;
    I : INTEGER;
BEGIN
  FNumIsdnInterfaces := 0;
  FNumCBIsdnInterfaces := 0;
  IF NOT i4ld_Connected THEN
    EXIT;
    //FIsdnInterfaceNames   : ARRAY [0..I4LMaxInterfacesK-1] OF STRING;

  // Get Interface List
  S := i4ld_Command ('list if');
  IF Copy (S,1,2) = 'Ok' THEN
  BEGIN
    Delete (S,1,3);
    I := Pos (' ', S);
    WHILE I > 0 DO
    BEGIN
      INC (FNumIsdnInterfaces);
      FIsdnInterfaceNames [FNumIsdnInterfaces-1] := Copy (S,1,I-1);
      Delete (S,1,I);
      I := Pos (' ', S);
    END;
    IF S <> '' THEN
    BEGIN
      INC (FNumIsdnInterfaces);
      FIsdnInterfaceNames [FNumIsdnInterfaces-1] := Copy (S,1,I-1);
    END;
  END;

  // Get Interfaces with Channel-Bundling
  S := i4ld_Command ('list if2');
  IF Copy (S,1,2) = 'Ok' THEN
  BEGIN
    Delete (S,1,3);
    I := Pos (' ', S);
    WHILE I > 0 DO
    BEGIN
      INC (FNumCBIsdnInterfaces);
      FCBIsdnInterfaceNames [FNumCBIsdnInterfaces-1] := Copy (S,1,I-1);
      Delete (S,1,I);
      I := Pos (' ', S);
    END;
    IF S <> '' THEN
    BEGIN
      INC (FNumCBIsdnInterfaces);
      FCBIsdnInterfaceNames [FNumCBIsdnInterfaces-1] := Copy (S,1,I-1);
    END;
  END;

END;




FUNCTION TIsdn4Linux.GetInterfaceName (Index : INTEGER) : STRING;
BEGIN
  IF (Index > FNumIsdnInterfaces-1) OR (Index < 0) THEN
    RAISE (TIsdn4LinuxException.Create ('GetInterfaceName: Index nicht im zulssigen Bereich'));
  Result := FIsdnInterfaceNames [Index];
END;


FUNCTION TIsdn4Linux.GetCBInterfaceName (Index : INTEGER) : STRING;
BEGIN
  IF (Index > FNumCBIsdnInterfaces-1) OR (Index < 0) THEN
    RAISE (TIsdn4LinuxException.Create ('GetCBInterfaceName: Index nicht im zulssigen Bereich'));
  Result := FCBIsdnInterfaceNames [Index];
END;


FUNCTION TIsdn4Linux.GetInterfaceOutNumbers (Index : INTEGER) : STRING;
VAR Name : STRING;
BEGIN
  Name := GetInterfaceName (Index);
  Result := i4ld_GetOutgoingNumbers (Name);
END;

FUNCTION TIsdn4Linux.GetInterfaceInNumbers (Index : INTEGER) : STRING;
VAR Name : STRING;
BEGIN
  Name := GetInterfaceName (Index);
  Result := i4ld_GetIncomingNumbers (Name);
END;


FUNCTION TIsdn4Linux.GetInterfaceDialmode (Index : INTEGER) : STRING;
VAR Name : STRING;
BEGIN
  Name := GetInterfaceName (Index);
  Result := i4ld_GetDialmode (Name);
END;



PROCEDURE TIsdn4Linux.SetInterfaceDialmode (Index : INTEGER; Dialmode : STRING);
VAR Name : STRING;
    Resu : STRING;
BEGIN
  Name := GetInterfaceName (Index);
  Resu := i4ld_SetDialmode (Name, Dialmode);
  IF Copy (Resu,1,2) <> 'Ok' THEN
    RAISE (TIsdn4LinuxException.Create ('SetInterfaceDialmode: '+Resu));
END;




FUNCTION TIsdn4Linux.GetInterfaceCount : INTEGER;
BEGIN
  Result := 0;
  IF NOT i4ld_Connected THEN
    EXIT;
  IF FNumIsdnInterfaces = 0 THEN i4ld_GetInterfaces;
  Result := FNumIsdnInterfaces;
END;


FUNCTION TIsdn4Linux.GetCBInterfaceCount : INTEGER;
BEGIN
  Result := 0;
  IF NOT i4ld_Connected THEN
    EXIT;
  IF FNumCBIsdnInterfaces = 0 THEN i4ld_GetInterfaces;
  Result := FNumCBIsdnInterfaces;
END;

// dials interface IpppName
PROCEDURE TIsdn4Linux.Dial (CONST IfName : STRING);
VAR S : STRING;
BEGIN
  IF NOT i4ld_Connected THEN
    RAISE (TIsdn4LinuxException.Create (i4ld_NotConnectedK));

  IF IfName = '' THEN
    RAISE (TIsdn4LinuxException.Create (i4ld_NoInterfaceK));

  S := i4ld_Command ('dial '+IfName);
  IF Copy (S,1,2) <> 'Ok' THEN
    RAISE (TIsdn4LinuxException.Create ('Dial:'+S));
END;

// hangup interface IpppName
PROCEDURE TIsdn4Linux.Hangup (CONST IfName : STRING);
VAR S : STRING;
BEGIN
  IF NOT i4ld_Connected THEN
    RAISE (TIsdn4LinuxException.Create ('i4ld_NotConnectedK'));

  IF IfName = '' THEN
    RAISE (TIsdn4LinuxException.Create (i4ld_NoInterfaceK));

  S := i4ld_Command ('hangup '+IfName);
  IF Copy (S,1,2) <> 'Ok' THEN
    RAISE (TIsdn4LinuxException.Create ('Hangup:'+S));
END;

// connect via tcp to i4lmond
FUNCTION TIsdn4Linux.i4ld_Connect : BOOLEAN;
VAR Err : INTEGER;
    S   : STRING;
BEGIN
  Result := i4ld_Connected;
  IF Result THEN EXIT;

  Err := TCPConnectTo (i4ld_SockHandle,PChar(FLinuxAddrOrName),
                       FLinuxPort+1,TRUE);
  IF Err = WSANOTINITIALISED THEN
  BEGIN
    InitWinSock;
    Err := TCPConnectTo (i4ld_SockHandle,PChar(FLinuxAddrOrName),
                         FLinuxPort+1,TRUE);
  END;
  IF Err <> 0 THEN
  BEGIN
    i4ld_SockHandle := -1;
    Result := FALSE;
    EXIT;
  END;
  // wait for READY
  IF NOT ReadALine (i4ld_SockHandle, S) THEN
  BEGIN
    CloseSocket (i4ld_SockHandle);
    i4ld_SockHandle := -1;
    Result := FALSE;
    EXIT;
  END;

  IF Copy (S,1,5) <> 'READY' THEN
  BEGIN
    CloseSocket (i4ld_SockHandle);
    i4ld_SockHandle := -1;
    Result := FALSE;
    EXIT;
  END;
  fi4ld_Version := Copy (S,7,255);
  Err := Pos (' ',fi4ld_Version);
  IF Err > 0 THEN
    Delete (fi4ld_Version,1,Err);
  Result := TRUE;
END;

// disconnect from i4lmond
FUNCTION TIsdn4Linux.i4ld_DisConnect : BOOLEAN;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  // Damit der i4lmond beenden wird, schicken wir ein Quit
  i4ld_Command ('quit 0');
  closeSocket (i4ld_SockHandle);
  i4ld_SockHandle := -1;
  fi4ld_Version := '';
  Result := TRUE;
END;

// are we connected to i4lmond ?
FUNCTION TIsdn4Linux.i4ld_Connected : BOOLEAN;
BEGIN
  Result := (i4ld_SockHandle <> -1);
END;


// Test Connection via Version-Command
FUNCTION TIsdn4Linux.i4ld_TestConnected : BOOLEAN;
VAR St : STRING;
BEGIN
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  St := i4ld_Command ('ver');
  IF Copy (St,1,2) <> 'Ok' THEN
  BEGIN
    closesocket (i4ld_SockHandle);
    i4ld_SockHandle := -1;
    Result := FALSE;
  END ELSE
    Result := TRUE;
END;


// Fill info into StGrid
FUNCTION TIsdn4Linux.i4ld_ListIf (CONST IfName : STRING;
                                        StGrid : TStringGrid) : BOOLEAN;
VAR St,St1 : STRING;
    P      : INTEGER;
    L      : INTEGER;
BEGIN
  StGrid.RowCount := 0;
  Result := i4ld_Connected;
  IF NOT Result THEN EXIT;

  StGrid.ColWidths[0] := 180;

  Result := FALSE;
  St1 := i4ld_Command ('list '+IfName);
  L := 0;
  WHILE (Copy (St1,1,6) <> '!ERROR') AND (Copy (St1,1,2) <> 'Ok') DO
  BEGIN
    St := St1;
    IF St <> '' THEN
    BEGIN
      Result := TRUE;
      P := Pos (':', St);
      StGrid.RowCount := StGrid.RowCount + 1;
      IF P = 0 THEN
      BEGIN
        StGrid.Cells [0,L] := St;
        StGrid.Cells [1,L] := '';
      END ELSE
      BEGIN
        StGrid.Cells [0,L] := Copy (St,1,P-1);
        Delete (St,1,P); St := Trim(St);
        StGrid.Cells [1,L] := St;
      END;
      INC (L);
    END;
    IF NOT ReadALine (i4ld_SockHandle, St1) THEN
      St1 := '!ERROR';
  END;
END;


PROCEDURE TIsdn4Linux.MarkAllDriverUnloaded;
BEGIN
  FillChar (foldDriverLoaded, SIZEOF (foldDriverLoaded), 0);
END;


FUNCTION TIsdn4Linux.i4ld_GetOutgoingNumbers (CONST IfName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('numout '+IfName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


//TInterfaceConnectStatus = (UnknownK, ConnectedInK, ConnectedOutK);
FUNCTION TIsdn4Linux.i4ld_GetInterfaceConnectionStatus (CONST IfName : STRING) : TInterfaceConnectStatus;
VAR S : STRING;
BEGIN
  Result := IFSUnknownK;
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  S := i4ld_Command ('status '+IfName);
  IF Copy (S,1,2) = 'Ok' THEN
  BEGIN
    IF Copy (S,4,1) = '0' THEN        // this is Null
      Result := IFSNotConnectedK
    ELSE
      IF Copy (S, 6, 1) = 'O' THEN    // and this [O]utgoing
        Result := IFSConnectedOutK
      ELSE
        Result := IFSConnectedInK;
  END;
END;



FUNCTION TIsdn4Linux.i4ld_GetIncomingNumbers (CONST IfName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('numin '+IfName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.i4ld_GetDialmode (CONST IfName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('dialmode '+IfName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.i4ld_SetDialmode (CONST IfName, Dialmode : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('dialmode '+IfName+' '+Dialmode);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.i4ld_DelNumber (CONST IfName, Number : STRING; Outgoing : BOOLEAN) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  IF Outgoing THEN
    Result := i4ld_Command ('delphoneout '+IfName+' '+Number)
  ELSE
    Result := i4ld_Command ('delphonein '+IfName+' '+Number);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;



FUNCTION TIsdn4Linux.i4ld_AddNumber (CONST IfName, Number : STRING; Outgoing : BOOLEAN) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  IF Outgoing THEN
    Result := i4ld_Command ('addphoneout '+IfName+' '+Number)
  ELSE
    Result := i4ld_Command ('addphonein '+IfName+' '+Number);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.i4ld_AddLink  (CONST IfName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('addlink '+IfName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.i4ld_RemoveLink (CONST IfName : STRING) : STRING;
BEGIN
  Result := '';
  IF NOT i4ld_Connected THEN EXIT;

  IF IfName = '' THEN
    EXIT;

  Result := i4ld_Command ('removelink '+IfName);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;




FUNCTION TIsdn4Linux.GetPhonebookEntry (Number : STRING) : STRING;
BEGIN
  Result := Number;
  IF NOT i4ld_Connected THEN EXIT;

  IF Number = '' THEN
    EXIT;

  Result := i4ld_Command ('getphonebook '+Number);
  IF Copy (Result,1,2) = 'Ok' THEN
    Result := Trim (Copy (Result,3,255));
END;


FUNCTION TIsdn4Linux.GetInterfaceNumber (Name : STRING) : INTEGER;
VAR I : INTEGER;
BEGIN
  Result := -1;
  IF NOT i4ld_Connected THEN EXIT;

  IF Name = '' THEN
    EXIT;
  FOR I := 0 TO FNumIsdnInterfaces-1 DO
    IF FIsdnInterfaceNames [I] = Name THEN
    BEGIN
      Result := I;
      EXIT;
    END;
END;


PROCEDURE TIsdn4Linux.SetErrorText (_MSG_UnableToConnect : STRING);
BEGIN
  MSG_UnableToConnect := _MSG_UnableToConnect;
END;


PROCEDURE Register;
BEGIN
  RegisterComponents('Extras', [TIsdn4Linux]);
END;

END.
